#!/usr/bin/env python

'''
This module implements equivalents of the basic ComputedEntry objects, which
is the basic entity that can be used to perform many analyses. ComputedEntries
contain calculated information, typically from VASP or other electronic structure
codes. For example, ComputedEntries can be used as inputs for phase diagram analysis.
'''

__author__ = "Shyue Ping Ong, Anubhav Jain"
__copyright__ = "Copyright 2011, The Materials Project"
__version__ = "1.0"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__status__ = "Production"
__date__ = "Sep 23, 2011M"

import json

from pymatgen.phasediagram.entries import PDEntry
from pymatgen.core.structure import Structure, Composition

class ComputedEntry(PDEntry):
    """
    An lightweight ComputedEntry object containing key computed data
    for many purposes.  Extends a PDEntry so that it can be used for phase diagram
    generation.
    
    Author: Shyue Ping Ong, Anubhav Jain
    """

    def __init__(self, composition, energy, correction = 0.0, parameters = None, data = None):
        """
        Args:
            composition:
                Composition of the entry. For flexibility, this can take the form
                of a {symbol: amt} dictionary as well the standard pymatgen
                Composition object.
            energy:
                Energy of the entry. Usually the final calculated energy from
                VASP or other electronic structure codes.
            correction:
                A correction to be applied to the energy. This is used to modify
                the energy for certain analyses. Defaults to 0.0.
            parameters:
                An optional dict of parameters associated with the entry. Defaults
                to None.
            data:
                An optional dict of any additional data associated with 
                the entry. Defaults to None
        """
        if not isinstance(composition, Composition):
            comp = Composition.from_dict(composition)
        else:
            comp = composition
        super(ComputedEntry, self).__init__(comp, energy)
        self.correction = correction
        self.parameters = parameters if parameters else {}
        self.data = data if data else {}

    @property
    def energy(self):
        """
        Returns the *corrected* energy of the entry.
        """
        return self.uncorrected_energy + self.correction

    @property
    def uncorrected_energy(self):
        """
        Returns the *uncorrected* energy of the entry.
        """
        return super(ComputedEntry, self).energy

    def __repr__(self):
        return "ComputedEntry {} with energy = {:.4f}, correction = {:.4f}".format(self.composition.formula, self.uncorrected_energy, self.correction)

    def __str__(self):
        outputstr = [self.__repr__()]
        outputstr.append("Parameters:")
        for k, v in self.parameters.items():
            outputstr.append("{} = {}".format(k, v))
        outputstr.append("Data:")
        for k, v in self.data.items():
            outputstr.append("{} = {}".format(k, v))
        return "\n".join(outputstr)

    @staticmethod
    def from_dict(d):
        return ComputedEntry(d['composition'], d['energy'], d['correction'],
                             d['parameters'], d['data'])

    @property
    def to_dict(self):
        d = dict()
        d['energy'] = self.uncorrected_energy
        d['composition'] = self.composition.to_dict
        d['correction'] = self.correction
        d['parameters'] = self.parameters
        d['data'] = self.data
        return d

class ComputedStructureEntry(ComputedEntry):
    """
    A heavier version of ComputedEntry which contains a structure as well. The 
    structure is needed for some analyses.
    
    Author: Shyue Ping Ong, Anubhav Jain
    """

    def __init__(self, structure, energy, correction = 0.0, parameters = None, data = None):
        """
        Args:
            structure:
                The actual structure of an entry.
            energy:
                Energy of the entry. Usually the final calculated energy from
                VASP or other electronic structure codes.
            correction:
                A correction to be applied to the energy. This is used to modify
                the energy for certain analyses. Defaults to 0.0.
            parameters:
                An optional dict of parameters associated with the entry. Defaults
                to None.
            data:
                An optional dict of any additional data associated with 
                the entry. Defaults to None
        """
        super(ComputedStructureEntry, self).__init__(structure.composition, energy, correction = correction, parameters = parameters, data = data)
        self.structure = structure

    def __repr__(self):
        return "ComputedStructureEntry {} with energy = {:.4f}, correction = {:.4f}".format(self.composition.formula, self.uncorrected_energy, self.correction)

    def __str__(self):
        outputstr = [self.__repr__()]
        outputstr.append("Parameters:")
        for k, v in self.parameters.items():
            outputstr.append("{} = {}".format(k, v))
        outputstr.append("Data:")
        for k, v in self.data.items():
            outputstr.append("{} = {}".format(k, v))
        return "\n".join(outputstr)

    @property
    def to_dict(self):
        d = super(ComputedStructureEntry, self).to_dict
        d['structure'] = self.structure.to_dict
        return d

    @staticmethod
    def from_dict(d):
        return ComputedStructureEntry(Structure.from_dict(d['structure']), d['energy'],
                             d['correction'], d['parameters'], d['data'])


def computed_entries_to_json(all_entries):
    jsonlist = list()
    for entry in all_entries:
        jsonlist.append(entry.to_dict)
    return json.dumps(jsonlist)

def computed_entries_from_json(jsonstr):
    json_list = json.loads(jsonstr)
    entries = list()
    for d in json_list:
        if 'structure' in d:
            entries.append(ComputedStructureEntry.from_dict(d))
        else:
            entries.append(ComputedEntry.from_dict(d))
    return entries



#!/usr/bin/env python

'''
This module implements input and output processing from molecules to Gaussian
Input files.
'''

from __future__ import division

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2012, The Materials Project"
__version__ = "0.1"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__date__ = "Apr 17, 2012"

import re
import math

from pymatgen.core.operations import SymmOp
import numpy as np
from pymatgen.core.structure import Molecule

class GaussianInput(object):
    """
    An object representing a Gaussian input file.
    """
    def __init__(self, mol, charge=0, spin_multiplicity=1, title=None,
                 functional="HF", basis_set="6-31G(d)", route_parameters=None,
                 input_parameters=None):
        """
        Args:
            mol:
                Input molecule
            charge:
                Charge of the molecule. Defaults to 0.
            spin_multiplicity:
                Spin multiplicity of molecule. Defaults to 1.
            title:
                Title for run. Defaults to formula of molecule if None.
            functional:
                Functional for run.
            basis_set:
                Basis set for run.
            route_parameters:
                Additional route parameters as a dict. For example, 
                {'SP':"", "SCF":"Tight"}
            input_parameters:
                Additional input parameters for run as a dict. Used for example,
                in PCM calculations.  E.g., {"EPS":12}
        """
        self._mol = mol
        self.charge = charge
        self.spin_multiplicity = spin_multiplicity
        self.functional = functional
        self.basis_set = basis_set
        self.route_parameters = route_parameters if route_parameters else {}
        self.input_parameters = input_parameters if input_parameters else {}
        self.title = title if title else self._mol.composition.formula

    @property
    def molecule(self):
        """
        Returns molecule associated with this GaussianInput.
        """
        return self._mol

    @staticmethod
    def parse_coords(coord_lines):
        """
        Helper method to parse coordinates.
        """
        paras = {}
        var_pattern = re.compile("^\s*([A-Za-z]+\S*)[\s=,]+([\d\-\.]+)\s*$")
        for l in coord_lines:
            m = var_pattern.match(l.strip())
            if m:
                paras[m.group(1)] = float(m.group(2))
        zmat_patt = re.compile("^\s*([A-Za-z]+)[\w\d\-\_]*([\s,]+(\w+)[\s,]+(\w+))*[\-\.\s,\w]*$")
        mixed_species_patt = re.compile("([A-Za-z]+)[\d\-\_]+")
        xyz_patt = re.compile("^\s*([A-Za-z]+[\w\d\-\_]*)\s+([\d\.eE\-]+)\s+([\d\.eE\-]+)\s+([\d\.eE\-]+)[\-\.\s,\w.]*$")

        parsed_species = []
        species = []
        coords = []

        for l in coord_lines:
            l = l.strip()
            if l == "":
                break
            if xyz_patt.match(l):
                m = xyz_patt.match(l)
                m2 = mixed_species_patt.match(m.group(1))
                if m2:
                    parsed_species.append(m.group(1))
                    species.append(m2.group(1))
                else:
                    species.append(m.group(1))

                toks = re.split("[,\s]+", l.strip())
                if len(toks) > 4:
                    coords.append(map(float, toks[2:5]))
                else:
                    coords.append(map(float, toks[1:4]))
            elif zmat_patt.match(l):
                toks = re.split("[,\s]+", l.strip())
                m = mixed_species_patt.match(toks[0])
                if m:
                    parsed_species.append(toks[0])
                    species.append(m.group(1))
                else:
                    species.append(toks[0])
                toks.pop(0)
                if len(toks) == 0:
                    coords.append(np.array([0, 0, 0]))
                else:
                    nn = []
                    parameters = []
                    while len(toks) > 0:
                        ind = toks.pop(0)
                        data = toks.pop(0)
                        try:
                            int(ind)
                            nn.append(int(ind))
                        except:
                            nn.append(parsed_species.index(ind))
                        parameters.append(paras[data])
                    if len(nn) == 1:
                        coords.append(np.array([0, 0, parameters[0]]))
                    elif len(nn) == 2:
                        coords1 = coords[nn[0] - 1]
                        coords2 = coords[nn[1] - 1]
                        bl = parameters[0]
                        angle = parameters[1]
                        axis = [0, 1, 0]
                        coord = SymmOp.from_origin_axis_angle(coords1, axis, angle, False).operate(coords2)
                        vec = coord - coords1
                        coord = vec * bl / np.linalg.norm(vec) + coords1
                        coords.append(coord)
                    elif len(nn) == 3:
                        coords1 = coords[nn[0] - 1]
                        coords2 = coords[nn[1] - 1]
                        coords3 = coords[nn[2] - 1]
                        bl = parameters[0]
                        angle = parameters[1]
                        dih = parameters[2]
                        v1 = coords3 - coords2
                        v2 = coords1 - coords2
                        axis = np.cross(v1, v2)
                        coord = SymmOp.from_origin_axis_angle(coords1, axis, angle, False).operate(coords2)
                        v1 = coord - coords1
                        v2 = coords1 - coords2
                        v3 = np.cross(v1, v2)
                        d = np.dot(v3, axis) / np.linalg.norm(v3) / np.linalg.norm(axis)
                        if d > 1:
                            d = 1
                        elif d < -1:
                            d = -1
                        adj = math.acos(d) * 180 / math.pi
                        axis = coords1 - coords2
                        coord = SymmOp.from_origin_axis_angle(coords1, axis, dih - adj, False).operate(coord)
                        vec = coord - coords1
                        coord = vec * bl / np.linalg.norm(vec) + coords1
                        coords.append(coord)

        return Molecule(species, coords)

    @staticmethod
    def from_string(contents):
        """
        Creates GaussianInput from a string.
        
        Args:
            contents:
                String representing an Gaussian input file.
        
        Returns:
            GaussianInput object
        """
        lines = contents.split("\n")
        route_patt = re.compile("^#[sSpPnN]*.*")

        route = None
        for i, l in enumerate(lines):
            if route_patt.match(l):
                route = l
                route_index = i
                break
        route_paras = {}
        if route:
            for tok in re.split("\s+", route):
                if tok.strip().startswith("#"):
                    continue
                if re.match("\w+\/.*", tok):
                    d = tok.split("/")
                    functional = d[0]
                    basis_set = d[1]
                else:
                    d = tok.split("=")
                    v = None if len(d) == 1 else d[1]
                    route_paras[d[0]] = v
        title = lines[route_index + 2]
        toks = re.split("\s+", lines[route_index + 4])
        charge = int(toks[0])
        spin_mult = int(toks[1])
        coord_lines = []
        spaces = 0
        input_paras = {}
        for i in xrange(route_index + 5, len(lines)):
            if lines[i].strip() == "":
                spaces += 1
            if spaces >= 2:
                d = lines[i].split("=")
                if len(d) == 2:
                    input_paras[d[0]] = float(d[1])
            else:
                coord_lines.append(lines[i].strip())
        mol = GaussianInput.parse_coords(coord_lines)

        return GaussianInput(mol, charge=charge, spin_multiplicity=spin_mult,
                             title=title, functional=functional,
                            basis_set=basis_set, route_parameters=route_paras,
                            input_parameters=input_paras)

    @staticmethod
    def from_file(filename):
        """
        Creates GaussianInput from a file.
        
        Args:
            filename:
                Gaussian input filename
                
        Returns:
            GaussianInput object
        """
        with open(filename, "r") as f:
            return GaussianInput.from_string(f.read())

    def _find_nn_pos_before_site(self, siteindex):
        """
        Returns index of nearest neighbor atoms.
        """
        alldist = [(self._mol.get_distance(siteindex, i), i) for i in xrange(siteindex)]
        alldist = sorted(alldist, key=lambda x:x[0])
        return [d[1] for d in alldist]

    def get_zmatrix(self):
        """
        Returns a z-matrix representation of the molecule.
        """
        output = []
        outputvar = []
        for i, site in enumerate(self._mol):
            if i == 0:
                output.append("{}".format(site.specie))
            elif i == 1:
                nn = self._find_nn_pos_before_site(i)
                bondlength = self._mol.get_distance(i, nn[0])
                output.append("{} {} B{}".format(self._mol[i].specie, nn[0] + 1, i))
                outputvar.append("B{}={:.6f}".format(i, bondlength))
            elif i == 2:
                nn = self._find_nn_pos_before_site(i)
                bondlength = self._mol.get_distance(i, nn[0])
                angle = self._mol.get_angle(i, nn[0], nn[1])
                output.append("{} {} B{} {} A{}".format(self._mol[i].specie, nn[0] + 1, i, nn[1] + 1, i))
                outputvar.append("B{}={:.6f}".format(i, bondlength))
                outputvar.append("A{}={:.6f}".format(i, angle))
            else:
                nn = self._find_nn_pos_before_site(i)
                bondlength = self._mol.get_distance(i, nn[0])
                angle = self._mol.get_angle(i, nn[0], nn[1])
                dih = self._mol.get_dihedral(i, nn[0], nn[1], nn[2])
                output.append("{} {} B{} {} A{} {} D{}".format(self._mol[i].specie, nn[0] + 1, i, nn[1] + 1, i, nn[2] + 1, i))
                outputvar.append("B{}={:.6f}".format(i, bondlength))
                outputvar.append("A{}={:.6f}".format(i, angle))
                outputvar.append("D{}={:.6f}".format(i, dih))
        return "\n".join(output) + "\n\n" + "\n".join(outputvar)

    def __str__(self):
        def para_dict_to_string(para, joiner=" "):
            para_str = []
            for k, v in para.items():
                if v:
                    para_str.append("{}={}".format(k, v))
                else:
                    para_str.append(k)
            return joiner.join(para_str)

        output = ["#P {func}/{bset} {route} Test".format(func=self.functional, bset=self.basis_set, route=para_dict_to_string(self.route_parameters))]
        output.append("")
        output.append(self.title)
        output.append("")
        output.append("{} {}".format(self.charge, self.spin_multiplicity))
        output.append(self.get_zmatrix())
        output.append("")
        output.append(para_dict_to_string(self.input_parameters, "\n"))
        output.append("")
        return "\n".join(output)




#!/usr/bin/env python

"""
This module provides classes that define a chemical reaction.
"""

from __future__ import division

__author__ = "Shyue Ping Ong, Anubhav Jain"
__copyright__ = "Copyright 2011, The Materials Project"
__version__ = "1.0"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__status__ = "Production"
__date__ = "$Sep 23, 2011M$"

import logging
import itertools
import numpy as np
from collections import defaultdict

from pymatgen.core.structure import Composition

logger = logging.getLogger(__name__)

class Reaction(object):
    """
    A class representing a Reaction.
    """

    TOLERANCE = 1e-6 #: Tolerance for determining if a particular component fraction is > 0.

    def __init__(self, reactants, products):
        """
        Reactants and products to be specified as list of 
        pymatgen.core.structure.Composition.  e.g., [comp1, comp2]
        
        Args:
            reactants: 
                List of reactants.
            products:
                List of products.
        """
        all_comp = reactants[:]
        all_comp.extend(products[:])
        els = set()
        for c in all_comp:
            els.update(c.elements)
        els = tuple(els)

        num_constraints = len(all_comp)
        num_els = len(els)
        dim = max(num_els, num_constraints)
        logger.debug('num_els = {}'.format(num_els))
        logger.debug('num_constraints = {}'.format(num_constraints))
        logger.debug('dim = {}'.format(dim))

        if num_constraints < 2:
            raise ReactionError("A reaction cannot be formed with just one composition.")
        elif num_constraints == 2:
            if all_comp[0].reduced_formula != all_comp[1].reduced_formula:
                raise ReactionError("%s and %s cannot be matched.  Reaction cannot be balanced. " % (all_comp[0].formula, all_comp[1].formula))
            else:
                coeffs = [-all_comp[1][els[0]] / all_comp[0][els[0]], 1]
        else:
            comp_matrix = np.zeros((dim, dim))
            count = 0
            if num_constraints < num_els:
                for i in range(num_constraints, num_els):
                    all_comp.append(Composition({els[i]:1}))
            for c in all_comp:
                for i in range(num_els):
                    comp_matrix[i][count] = c[els[i]]
                count += 1

            if num_constraints > num_els:

                #Try two schemes for making the comp matrix non-singular.
                for i in range(num_els, num_constraints):
                    for j in range(num_els):
                        comp_matrix[i][j] = 0
                    comp_matrix[i][i] = 1
                count = 0
                if abs(np.linalg.det(comp_matrix)) < self.TOLERANCE:
                    for i in range(num_els, num_constraints):
                        for j in range(num_els):
                            comp_matrix[i][j] = count
                            count += 1
                        comp_matrix[i][i] = count
                ans_matrix = np.zeros(num_constraints)
                ans_matrix[num_els:num_constraints] = 1
                coeffs = np.linalg.solve(comp_matrix, ans_matrix)
            else:
                if abs(np.linalg.det(comp_matrix)) < self.TOLERANCE:
                    logger.debug('Linear solution possible. Trying various permutations.')
                    comp_matrix = comp_matrix[0:num_els][:, 0:num_constraints]
                    logger.debug('comp_matrix = {}'.format(comp_matrix))
                    ans_found = False
                    for perm_matrix in itertools.permutations(comp_matrix):
                        logger.debug('Testing permuted matrix = {}'.format(perm_matrix))
                        for m in xrange(num_constraints):
                            submatrix = [[perm_matrix[i][j] for j in xrange(num_constraints) if j != m] for i in xrange(num_constraints) if i != m]
                            logger.debug('Testing submatrix = {}'.format(submatrix))
                            if abs(np.linalg.det(submatrix)) > self.TOLERANCE:
                                logger.debug('Possible sol')
                                subansmatrix = [perm_matrix[i][m] for i in xrange(num_constraints) if i != m]
                                coeffs = -np.linalg.solve(submatrix, subansmatrix)
                                coeffs = [c for c in coeffs]
                                coeffs.insert(m, 1)
                                #Check if final coeffs are valid
                                overall_mat = np.dot(perm_matrix, coeffs)
                                if (abs(overall_mat) < 1e-8).all():
                                    ans_found = True
                                    break
                    if not ans_found:
                        raise ReactionError("Reaction is ill-formed and cannot be balanced.")
                else:
                    raise ReactionError("Reaction is ill-formed and cannot be balanced.")

        for i in xrange(len(coeffs) - 1, -1, -1):
            if coeffs[i] != 0:
                normfactor = coeffs[i]
                break
        #Invert negative solutions and scale to final product
        coeffs = [c / normfactor for c in coeffs]
        self._els = els
        self._all_comp = all_comp[0:num_constraints]
        self._coeffs = coeffs[0:num_constraints]
        self._num_comp = num_constraints

    def copy(self):
        """
        Returns a copy of the Reaction object.
        """
        return Reaction(self.reactants, self.products)

    def calculate_energy(self, energies):
        """
        Calculates the energy of the reaction.
         
        Args:
            energies - dict of {comp:energy}.  e.g., {comp1: energy1, comp2: energy2}.
        
        Returns:
            reaction energy as a float.
        """
        return sum([self._coeffs[i] * energies[self._all_comp[i]] for i in range(self._num_comp)])

    def normalize_to(self, comp, factor = 1):
        """
        Normalizes the reaction to one of the compositions.
        By default, normalizes such that the composition given has a coefficient of 1.
        Another factor can be specified.
        """
        scale_factor = abs(1 / self._coeffs[self._all_comp.index(comp)] * factor)
        self._coeffs = [c * scale_factor for c in self._coeffs]

    def normalize_to_element(self, element, target_amount = 1):
        """
        Normalizes the reaction to one of the elements.
        By default, normalizes such that the amount of the element is 1.
        Another factor can be specified.
        """
        current_element_amount = sum([self._all_comp[i][element] * abs(self._coeffs[i]) for i in xrange(len(self._all_comp))]) / 2
        scale_factor = target_amount / current_element_amount
        self._coeffs = [c * scale_factor for c in self._coeffs]

    def get_el_amount(self, element):
        return sum([self._all_comp[i][element] * abs(self._coeffs[i]) for i in xrange(len(self._all_comp))]) / 2

    @property
    def elements(self):
        """
        List of elements in the reaction
        """
        return self._els[:]

    @property
    def coeffs(self):
        """
        Final coefficients of the calculated reaction
        """
        return self._coeffs[:]

    @property
    def all_comp(self):
        """
        List of all compositions in the reaction.
        """
        return self._all_comp

    @property
    def reactants(self):
        """List of reactants"""

        return [self._all_comp[i] for i in xrange(len(self._all_comp)) if self._coeffs[i] < 0]

    @property
    def products(self):
        """List of products"""
        return [self._all_comp[i] for i in xrange(len(self._all_comp)) if self._coeffs[i] > 0]

    def get_coeff(self, comp):
        """Returns coefficient for a particular composition"""
        return self._coeffs[self._all_comp.index(comp)]

    def normalized_repr_and_factor(self):
        """
        Normalized representation for a reaction
        For example, ``4 Li + 2 O -> 2Li2O`` becomes ``2 Li + O -> Li2O``
        """
        reactant_str = []
        product_str = []
        scaled_coeffs = []
        reduced_formulas = []
        for i in range(self._num_comp):
            comp = self._all_comp[i]
            coeff = self._coeffs[i]
            (reduced_formula, scale_factor) = comp.get_reduced_formula_and_factor()
            scaled_coeffs.append(coeff * scale_factor)
            reduced_formulas.append(reduced_formula)

        count = 0
        while sum([abs(coeff) % 1 for coeff in scaled_coeffs]) > 1e-8:
            norm_factor = 1 / smart_float_gcd(scaled_coeffs)
            scaled_coeffs = [c / norm_factor for c in scaled_coeffs]
            count += 1
            if count > 10: #Prevent an infinite loop
                break

        for i in range(self._num_comp):
            if scaled_coeffs[i] == -1:
                reactant_str.append(reduced_formulas[i])
            elif scaled_coeffs[i] == 1:
                product_str.append(reduced_formulas[i])
            elif scaled_coeffs[i] < 0:
                reactant_str.append("%d %s" % (-scaled_coeffs[i], reduced_formulas[i]))
            elif scaled_coeffs[i] > 0:
                product_str.append("%d %s" % (scaled_coeffs[i], reduced_formulas[i]))
        factor = scaled_coeffs[0] / self._coeffs[0]

        return (" + ".join(reactant_str) + " -> " + " + ".join(product_str), factor)

    @property
    def normalized_repr(self):
        return self.normalized_repr_and_factor()[0]

    def __repr__(self):
        return self.__str__()

    def __str__(self):
        reactant_str = []
        product_str = []
        for i in range(self._num_comp):
            comp = self._all_comp[i]
            coeff = self._coeffs[i]
            red_comp = Composition.from_formula(comp.reduced_formula)
            scale_factor = comp.num_atoms / red_comp.num_atoms
            scaled_coeff = coeff * scale_factor
            if scaled_coeff < 0:
                reactant_str.append("%.3f %s" % (-scaled_coeff, comp.reduced_formula))
            elif scaled_coeff > 0:
                product_str.append("%.3f %s" % (scaled_coeff, comp.reduced_formula))

        return " + ".join(reactant_str) + " -> " + " + ".join(product_str)


def smart_float_gcd(list_of_floats):
    """
    Determines the great common denominator (gcd).  Works on floats as well as integers.
    
    Args:
        list_of_floats: List of floats to determine gcd.
    """
    mult_factor = 1.0
    all_remainders = sorted([abs(f - int(f)) for f in list_of_floats])
    for i in range(len(all_remainders)):
        if all_remainders[i] > 1e-5:
            mult_factor *= all_remainders[i]
            all_remainders = [ f2 / all_remainders[i] - int(f2 / all_remainders[i]) for f2 in all_remainders]
    return 1 / mult_factor


class ReactionError(Exception):
    '''
    Exception class for Reactions. Allows more information exception messages to cover situations not
    covered by standard exception classes.
    '''

    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return self.msg


class BalancedReaction(Reaction):
    """
    An extended version of Reaction to allow coefficients to be specified.
    """

    def __init__(self, reactants_coeffs, products_coeffs):
        """
        Reactants and products to be specified as dict of 
        pymatgen.core.structure.Composition : coeff.  
        
        Args:
            reactants : List of reactants.
            products : List of products.
        """
        coeffs = []
        all_comp = []
        for comp, c in reactants_coeffs.items():
            if comp not in all_comp:
                all_comp.append(comp)
                coeffs.append(-c)
            else:
                ind = all_comp.index(comp)
                coeffs[ind] += -c

        for comp, c in products_coeffs.items():
            if comp not in all_comp:
                all_comp.append(comp)
                coeffs.append(c)
            else:
                ind = all_comp.index(comp)
                coeffs[ind] += c
        els = set()
        for c in all_comp:
            els.update(c.elements)
        els = tuple(els)

        sum_comp = defaultdict(int)

        for i in xrange(len(all_comp)):
            for el in els:
                sum_comp[el] += coeffs[i] * all_comp[i][el]

        for v in sum_comp.values():
            if abs(v) > Reaction.TOLERANCE:
                raise ReactionError("Reaction is unbalanced with {}!".format(v))

        self._els = els
        self._all_comp = all_comp
        self._coeffs = coeffs
        self._num_comp = len(self._all_comp)


class ComputedReaction(Reaction):
    """
    Convenience class to generate a reaction from ComputedEntry objects, with
    some additional attributes, such as a reaction energy based on computed
    energies.
    """

    def __init__(self, reactant_entries, product_entries):
        """
        Args:
            reactant_entries: 
                List of reactant_entries.
            products: 
                List of product_entries.
        """
        self._reactant_entries = reactant_entries
        self._product_entries = product_entries
        reactant_comp = set([e.composition.get_reduced_composition_and_factor()[0] for e in reactant_entries])
        product_comp = set([e.composition.get_reduced_composition_and_factor()[0] for e in product_entries])
        super(ComputedReaction, self).__init__(list(reactant_comp), list(product_comp))

    @property
    def calculated_reaction_energy(self):
        calc_energies = {}
        def update_calc_energies(entry):
            (comp, factor) = entry.composition.get_reduced_composition_and_factor()
            if comp not in calc_energies:
                calc_energies[comp] = entry.energy / factor
            else:
                calc_energies[comp] = min(calc_energies[comp], entry.energy / factor)
        map(update_calc_energies, self._reactant_entries)
        map(update_calc_energies, self._product_entries)
        return self.calculate_energy(calc_energies)




#!/usr/bin/python

import unittest
import os
import json

from pymatgen.electronic_structure.band_structure.band_structure import Kpoint
from pymatgen.core.structure import Lattice
from pymatgen.electronic_structure.band_structure.band_structure import BandStructureSymmLine

module_dir = os.path.dirname(os.path.abspath(__file__))

class KpointTest(unittest.TestCase):

    def setUp(self):
        self.lattice = Lattice.cubic(10.0)
        self.kpoint = Kpoint([0.1, 0.4, -0.5], self.lattice, label = "X")

    def test_properties(self):
        self.assertEquals(self.kpoint.frac_coords[0], 0.1)
        self.assertEquals(self.kpoint.frac_coords[1], 0.4)
        self.assertEquals(self.kpoint.frac_coords[2], -0.5)
        self.assertEquals(self.kpoint.a, 0.1)
        self.assertEquals(self.kpoint.b, 0.4)
        self.assertEquals(self.kpoint.c, -0.5)
        self.assertEquals(self.lattice, Lattice.cubic(10.0))
        self.assertEquals(self.kpoint.cart_coords[0], 1.0)
        self.assertEquals(self.kpoint.cart_coords[1], 4.0)
        self.assertEquals(self.kpoint.cart_coords[2], -5.0)
        self.assertEqual(self.kpoint.label, "X")

class BandStructureSymmLine_test(unittest.TestCase):

    def setUp(self):
        with open(os.path.join(module_dir, "Cao_2605.json"), "rb") as f:
            d = json.loads(f.read())
            self.bs = BandStructureSymmLine.from_dict(d)
            #this doesn't really test to_dict -> from_dict very well
            self.assertEqual(self.bs.to_dict.keys(), d.keys())
            self.one_kpoint = self.bs.kpoints[31]
            self.assertEqual(self.bs._nb_bands, 14)
            self.assertAlmostEqual(self.bs._bands[5]['energy'][10], 0.5608)
            self.assertAlmostEqual(self.bs._bands[5]['energy'][10], 0.5608)
            self.assertEqual(self.bs._bands[5]['occup'][60], 1.0)
            self.assertEqual(self.bs._branches[5]['name'], "L-U")
            self.assertEqual(self.bs._branches[5]['start_index'], 80)
            self.assertEqual(self.bs._branches[5]['end_index'], 95)
            self.assertAlmostEqual(self.bs._distance[70], 4.2335127528765737)

    def test_properties(self):
        self.assertEqual(self.one_kpoint.frac_coords[0], 0.5)
        self.assertEqual(self.one_kpoint.frac_coords[1], 0.25)
        self.assertEqual(self.one_kpoint.frac_coords[2], 0.75)
        self.assertAlmostEqual(self.one_kpoint.cart_coords[0], 0.79508915)
        self.assertAlmostEqual(self.one_kpoint.cart_coords[1], 0.45904494)
        self.assertAlmostEqual(self.one_kpoint.cart_coords[2], 1.12442586)
        self.assertEqual(self.one_kpoint.label, "W")

        self.assertAlmostEqual(self.bs.efermi, 2.6211967, "wrong fermi energy")

    def test_get_branch_name(self):
        self.assertAlmostEqual(self.bs.get_branch_name(110)[0], "U-W")

    def test_is_metal(self):
        self.assertFalse(self.bs.is_metal(), "wrong metal assignment")

    def test_get_cbm(self):
        cbm = self.bs.get_cbm()
        self.assertAlmostEqual(cbm['energy'], 5.8709, "wrong CBM energy")
        self.assertEqual(cbm['band_index'][0], 8, "wrong CBM band index")
        self.assertEqual(cbm['kpoint_index'][0], 15, "wrong CBM kpoint index")
        self.assertEqual(cbm['kpoint'].frac_coords[0], 0.5, "wrong CBM kpoint frac coords")
        self.assertEqual(cbm['kpoint'].frac_coords[1], 0.0, "wrong CBM kpoint frac coords")
        self.assertEqual(cbm['kpoint'].frac_coords[2], 0.5, "wrong CBM kpoint frac coords")
        self.assertEqual(cbm['kpoint'].label, "X", "wrong CBM kpoint label")

    def test_get_vbm(self):
        vbm = self.bs.get_vbm()
        self.assertAlmostEqual(vbm['energy'], 2.2361, "wrong VBM energy")
        self.assertEqual(len(vbm['band_index']), 3, "wrong VBM number of bands")
        self.assertEqual(vbm['band_index'][0], 5, "wrong VBM band index")
        self.assertEqual(vbm['kpoint_index'][0], 0, "wrong VBM kpoint index")
        self.assertEqual(vbm['kpoint'].frac_coords[0], 0.0, "wrong VBM kpoint frac coords")
        self.assertEqual(vbm['kpoint'].frac_coords[1], 0.0, "wrong VBM kpoint frac coords")
        self.assertEqual(vbm['kpoint'].frac_coords[2], 0.0, "wrong VBM kpoint frac coords")
        self.assertEqual(vbm['kpoint'].label, "\Gamma", "wrong VBM kpoint label")

    def test_get_band_gap(self):
        bg = self.bs.get_band_gap()
        self.assertEqual(bg['energy'], 3.6348, "wrong gap energy")
        self.assertEqual(bg['transition'], "\\Gamma-X", "wrong kpoint transition")
        self.assertFalse(bg['direct'], "wrong nature of the gap")

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

'''
Defines an abstract base class contract for Transformation object.
'''

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2011, The Materials Project"
__version__ = "0.1"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__date__ = "Sep 23, 2011"


import abc
import json


class AbstractTransformation(object):
    """
    Abstract transformation class.    
    """
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def apply_transformation(self, structure):
        '''
        Applies the transformation to a structure. Depending on whether a
        transformation is one-to-many, there may be an option to return a
        ranked list of structures.
        
        Args:
            structure:
                input structure
            return_ranked_list:
                Boolean stating whether or not multiple structures are
                returned. If return_ranked_list is a number, that number of
                structures is returned.
                
        Returns:
            depending on returned_ranked list, either a transformed structure 
            or
            a list of dictionaries, where each dictionary is of the form 
            {'structure' = .... , 'other_arguments'}
            the key 'transformation' is reserved for the transformation that
            was actually applied to the structure. 
            This transformation is parsed by the alchemy classes for generating
            a more specific transformation history. Any other information will
            be stored in the transformation_parameters dictionary in the 
            transmuted structure class.
        '''
        return

    @abc.abstractproperty
    def inverse(self):
        '''
        Returns the inverse transformation if available.
        Otherwise, should return None.
        '''
        return

    @abc.abstractproperty
    def is_one_to_many(self):
        '''
        Determines if a Transformation is a one-to-many transformation. If a
        Transformation is a one-to-many transformation, the apply_transformation
        method should have a keyword arg "return_ranked_list" which allows for
        the transformed structures to be returned as a ranked list.
        '''
        return False

    def to_json(self):
        return json.dumps(self.to_dict)


    @abc.abstractproperty
    def to_dict(self):
        '''
        Creates a json representation of the transformation, which must contain
        all necessary information to allow a reconstruction from a from_json static method.
        Format should follow:
        {'name' : transformation_class_name, 'init_arguments' : (init arguments)}
        '''
        return



#!/usr/bin/env python

"""
A convenience script engine using VaspObjects to do all manner of simple outputs.
Written by Shyue Ping Ong
"""


import argparse
import os
import re

from multiprocessing import Manager, Pool

from pymatgen.io.vaspio import Vasprun, Outcar
from pymatgen.util.string_utils import str_aligned

def get_vasprun_energy(args):
    (rootdir, parent, f, all_data) = args
    fullpath = os.path.join(parent, f)
    displaydir = re.sub(re.escape(rootdir+'/'), "", fullpath)
    try:
        xmlrun = Vasprun(fullpath)
        energy = xmlrun.final_energy
        num_atoms = len(xmlrun.final_structure)            
        all_data.append((displaydir, energy, energy/num_atoms))
    except:
        all_data.append((displaydir, 'NA', 'NA'))

def get_energies(rootdir):
    manager = Manager()
    all_data = manager.list()
    all_args = []
    for (parent, subdirs, files) in os.walk(rootdir):
        for f in files:
            if re.match("vasprun.xml.*", f):
                all_args.append((rootdir,parent,f,all_data))
    p = Pool(8)
    p.map(get_vasprun_energy, all_args)
    all_data = sorted(all_data, key=lambda x:x[0])
    print str_aligned(all_data, ("Directory", "Energy", "Energy/Atom"))

def get_magnetizations(mydir, ionList):
    print "%10s | %7s" % ("Ion", "Magmoms")
    print "-" * 20
    for (parent, subdirs, files) in os.walk(mydir):
        for f in files:
            if re.match("OUTCAR*", f):
                fullpath = os.path.join(parent, f)
                outcar = Outcar(fullpath)
                mags = outcar.magnetization
                mags = [m['tot'] for m in mags]
                allIons = xrange(len(mags))
                print "%16s" % (fullpath.lstrip("./"))
                if len(ionList) > 0:
                    allIons = ionList
                for ion in allIons:
                    print "%10d | %3.4f" % (ion, mags[ion])
                print "-" * 20

parser = argparse.ArgumentParser(description='''Convenient vasp run analyzer which can recursively go into a directory to search results.
Author: Shyue Ping Ong
Version: 1.0
Last updated: Nov 15 2011''')
parser.add_argument('directories', metavar='dir', default='.', type=str, nargs = '*', help='directory to process (default to .)')
parser.add_argument('-e', '--energies', dest='get_energies', action='store_const', const=True, help='print energies')
parser.add_argument('-m', '--mag', dest="ion_list", type=str, nargs = 1, help='print magmoms. ION LIST can be a range (e.g., 1-2) or the string "All" for all ions.')

args = parser.parse_args()
if args.get_energies:
    map(get_energies, args.directories)
if args.ion_list:
    ion_list = list()
    (start, end) = map(int, re.split("-", args.ion_list[0]))
    ion_list = range(start, end + 1)
    get_magnetizations(args.directories, ion_list)



#!/usr/bin/env python

'''
This module define the various drones used to assimilate data.
'''

from __future__ import division

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2012, The Materials Project"
__version__ = "0.1"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__date__ = "Mar 18, 2012"

import abc
import os
import re
import glob
import logging

from pymatgen.io.vaspio import Vasprun
from pymatgen.entries.computed_entries import ComputedEntry, ComputedStructureEntry

logger = logging.getLogger(__name__)


class AbstractDrone(object):
    """
    Abstract drone class that defines the various methods that must be implemented
    by drones. Because of the quirky nature of Python's multiprocessing, the 
    representations has to be in the form of python primitives. This can then
    be reverted to the original object with drone.convert.
    """
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def assimilate(self, path):
        '''
        Assimilate data in a directory path into a dict representation of
        a pymatgen object. Because of the quirky nature of Python's
        multiprocessing, the representations has to be in the form of python
        primitives. Hence the object must supprot a to_dict. This can then
        be reverted to the original object with drone.convert.
        
        Args:
            path:
                directory path
                
        Returns:
            Dict representation of an assimilated object
        '''
        return

    @abc.abstractmethod
    def get_valid_paths(self, path):
        """
        Checks if path contains valid data for assimilation, and then returns
        the valid paths. The paths returned can be a list of directory or file
        paths, depending on what kind of data you are assimilating. For example,
        if you are assimilating VASP runs, you are only interested in
        directories containing vasprun.xml files. On the other hand, if you are
        interested converting all POSCARs in a directory tree to cifs for
        example, you will want the file paths.
        
        Args:
            path:
                input path as a tuple generated from os.walk, i.e.,
                (parent, subdirs, files).
                
        Returns:
            List of valid dir/file paths for assimilation
        """
        return

    @abc.abstractmethod
    def convert(self, d):
        """
        Convert a dict generated from assimilate into an actual pymatgen object.
        """

    @abc.abstractproperty
    def to_dict(self):
        """
        All drones must support a json serializable dict representation
        """
        return


class VaspToComputedEntryDrone(AbstractDrone):
    """
    VaspToEntryDrone assimilates directories containing vasp input to 
    ComputedEntry/ComputedStructureEntry objects. There are some restrictions
    on the valid directory structures:
    
    1. There can be only one vasp run in each directory.
    2. Directories designated "relax1", "relax2" are considered to be 2 parts of
       an aflow style run, and only "relax2" is parsed.
       
    """

    def __init__(self, inc_structure=False, parameters=None, data=None):
        """
        Args:
            inc_structure:
                Set to True if you want ComputedStructureEntries to be returned
                instead of ComputedEntries.
            parameters:
                Input parameters to include. It has to be one of the properties
                supported by the Vasprun object. See pymatgen.io.vaspio Vasprun.
                The parameters have to be one of python's primitive types,
                i.e. list, dict of strings and integers. Complex objects such as
                dos are not supported at this point.
                If parameters == None, a default set of parameters that are 
                necessary for typical post-processing will be set.
            data:
                Output data to include. Has to be one of the properties
                supported by the Vasprun object. The parameters have to be one
                of python's primitive types, i.e. list, dict of strings and
                integers. Complex objects such as dos are not supported at this
                point.
        """
        self._inc_structure = inc_structure
        self._parameters = parameters if parameters else ["is_hubbard", "hubbards", "potcar_symbols", "run_type"]
        self._data = data if data else []

    def assimilate(self, path):
        files = os.listdir(path)
        if 'relax1' in files and 'relax2' in files:
            filepath = glob.glob(os.path.join(path, "relax2", "vasprun.xml*"))[0]
        else:
            vasprun_files = glob.glob(os.path.join(path, "vasprun.xml*"))
            filepath = None
            if len(vasprun_files) == 1:
                filepath = vasprun_files[0]
            elif len(vasprun_files) > 1:
                """
                This is a bit confusing, since there maybe be multi-steps. By 
                default, assimilate will try to find a file simply named 
                vasprun.xml, vasprun.xml.bz2, or vasprun.xml.gz.  Failing which
                it will try to get a relax2 from an aflow style run if possible.
                Or else, a randomly chosen file containing vasprun.xml is chosen.
                """
                for fname in vasprun_files:
                    if os.path.basename(fname) in ["vasprun.xml", "vasprun.xml.gz", "vasprun.xml.bz2"]:
                        filepath = fname
                        break
                    if re.search("relax2", fname):
                        filepath = fname
                        break
                    filepath = fname

        try:
            vasprun = Vasprun(filepath)
        except Exception as ex:
            logger.debug("error in {}: {}".format(filepath, ex))
            return None
        param = {}
        for p in self._parameters:
            param[p] = getattr(vasprun, p)
        data = {}
        for d in self._data:
            data[d] = getattr(vasprun, d)
        if self._inc_structure:
            entry = ComputedStructureEntry(vasprun.final_structure,
                                   vasprun.final_energy, parameters=param, data=data)
        else:
            entry = ComputedEntry(vasprun.final_structure.composition,
                                   vasprun.final_energy, parameters=param, data=data)
        return entry.to_dict

    def get_valid_paths(self, path):
        (parent, subdirs, files) = path
        if 'relax1' in subdirs and 'relax2' in subdirs:
            return [parent]
        if (not parent.endswith('/relax1')) and (not parent.endswith('/relax2')) and len(glob.glob(os.path.join(parent, "vasprun.xml*"))) > 0:
            return [parent]
        return []

    def convert(self, d):
        if 'structure' in d:
            return ComputedStructureEntry.from_dict(d)
        else:
            return ComputedEntry.from_dict(d)

    def __str__(self):
        return "VaspToEntryDrone"

    @property
    def to_dict(self):
        init_args = {'inc_structure' : self._inc_structure,
                     "parameters": self._parameters,
                     "data": self._data}
        output = {'name' : self.__class__.__name__, 'init_args': init_args, 'version': __version__ }
        return output


def drone_from_dict(d):
    """
    A helper function that returns a drone from a dict representation.
    
    Arguments:
        d:
            A dict representation of a drone with init args.
    
    Returns:
        A properly initialized Drone object
    """
    mod = __import__('pymatgen.borg.hive', globals(), locals(), [d['name']], -1)
    if hasattr(mod, d['name']):
        drone = getattr(mod, d['name'])
        return drone(**d['init_args'])
    raise ValueError("Invalid Drone Dict")



#!/usr/bin/python

import unittest
import os
import pymatgen.electronic_structure.band_structure
from pymatgen.electronic_structure.band_structure.plotter import BSPlotter

module_dir = os.path.dirname(os.path.abspath(__file__))

class BSPlotterTest(unittest.TestCase):

    def setUp(self):
        import json
        with open(os.path.join(module_dir, "Cao_2605.json"), "rb") as f:
            d = json.loads(f.read())
            self.bs = pymatgen.electronic_structure.band_structure.band_structure.BandStructureSymmLine.from_dict(d)
            self.plotter = BSPlotter(self.bs)

    def test_bs_plot_data(self):
        self.assertEqual(len(self.plotter.bs_plot_data['distances']), 160, "wrong number of distances")
        self.assertEqual(self.plotter.bs_plot_data['ticks']['label'][5], "K", "wrong tick label")
        self.assertEqual(len(self.plotter.bs_plot_data['ticks']['label']), 19, "wrong number of tick labels")

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

"""
This module provides classes for calculating the ewald sum of a structure.
"""

from __future__ import division

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2011, The Materials Project"
__credits__ = "Christopher Fischer"
__version__ = "1.0"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__status__ = "Production"
__date__ = "$Sep 23, 2011M$"

import numpy as np
from math import pi, sqrt, log, exp, cos, sin, erfc
import scipy.constants as sc
from scipy.misc import comb
from pymatgen.core.structure import Structure
from copy import deepcopy, copy
import bisect


class EwaldSummation:
    """
    Calculates the electrostatic energy of a periodic array of charges using the Ewald technique. 
    References : http://www.ee.duke.edu/~ayt/ewaldpaper/ewaldpaper.html
    
    This matrix can be used to do fast calculations of ewald sums after species removal
    
    E = E_recip + E_real + E_point
    
    Atomic units used in the code, then converted to eV.
    """

    # taken from convasp. converts unit of q*q/r into eV 
    CONV_FACT = 1e10 * sc.e / (4 * pi * sc.epsilon_0)

    def __init__(self, structure, real_space_cut = -1.0, recip_space_cut = -1.0, eta = -1.0, acc_factor = 8.0):
        """
        Initializes and calculates the Ewald sum. Default convergence parameters have been
        specified, but you can override them if you wish.
        
        Args:
            structure: 
                input structure that must have proper Specie on all sites, i.e.
                Element with oxidation state. Use OxidationStateDecorator in 
                pymatgen.core.structure_modifier for example.
            real_space_cut: 
                Real space cutoff radius dictating how many terms are used in the 
                real space sum. negative means determine automagically using the
                formula given in gulp 3.1 documentation.
            recip_space_cut: 
                Reciprocal space cutoff radius. negative means determine automagically using
                formula given in gulp 3.1 documentation.
            eta: 
                The screening parameter. negative means determine automatically.
                calculate_forces: Set to true if forces are desired
            acc_factor: 
                No. of significant figures each sum is converged to. See the gulp manual. 
        """
        self._s = structure
        self._vol = structure.volume

        self._acc_factor = acc_factor

        # set screening length
        self._eta = eta if eta > 0 else (len(structure) * 0.01 / self._vol) ** (1 / 3) * pi
        self._sqrt_eta = sqrt(self._eta)

        # acc factor used to automatically determine the optimal real and 
        # reciprocal space cutoff radii
        self._accf = sqrt(log(10 ** acc_factor))

        self._rmax = real_space_cut if real_space_cut > 0 else self._accf / self._sqrt_eta
        self._gmax = recip_space_cut if recip_space_cut > 0 else 2 * self._sqrt_eta * self._accf

        """
        The next few lines pre-compute certain quantities and store them. Ewald
        summation is rather expensive, and these shortcuts are necessary to obtain
        several factors of improvement in speedup.
        """
        self._oxi_states = [compute_average_oxidation_state(site) for site in structure]
        self._coords = np.array(self._s.cart_coords)
        self._forces = np.zeros((len(structure), 3))

        """
        Now we call the relevant private methods to calculate the reciprocal
        and real space terms.
        """
        (self._recip, recip_forces) = self._calc_recip()
        (self._real, self._point, real_point_forces) = self._calc_real_and_point()
        self._forces = recip_forces + real_point_forces

    def compute_partial_energy(self, removed_indices):
        """
        Gives total ewald energy for certain sites being removed, i.e. zeroed
        out. 
        """
        total_energy_matrix = self.total_energy_matrix.copy()
        for i in removed_indices:
            total_energy_matrix[i, :] = 0
            total_energy_matrix[:, i] = 0
        return sum(sum(total_energy_matrix))

    @property
    def reciprocal_space_energy(self):
        return sum(sum(self._recip))

    @property
    def reciprocal_space_energy_matrix(self):
        return self._recip

    @property
    def real_space_energy(self):
        return sum(sum(self._real))

    @property
    def real_space_energy_matrix(self):
        return self._real

    @property
    def point_energy(self):
        return sum(self._point)

    @property
    def point_energy_matrix(self):
        return self._point

    @property
    def total_energy(self):
        return sum(sum(self._recip)) + sum(sum(self._real)) + sum(self._point)

    @property
    def total_energy_matrix(self):
        totalenergy = self._recip + self._real
        for i in range(len(self._point)):
            totalenergy[i, i] += self._point[i]
        return totalenergy

    @property
    def forces(self):
        return self._forces

    def _calc_recip(self):
        """
        Perform the reciprocal space summation. Calculates the quantity
        E_recip = 1/(2PiV) sum_{G < Gmax} exp(-(G.G/4/eta))/(G.G) S(G)S(-G) where
        S(G) = sum_{k=1,N} q_k exp(-i G.r_k)
        S(G)S(-G) = |S(G)|**2
        
        This method is heavily vectorized to utilize numpy's C backend for speed.
        """
        numsites = self._s.num_sites
        prefactor = 2 * pi / self._vol
        erecip = np.zeros((numsites, numsites))
        forces = np.zeros((numsites, 3))
        coords = self._coords
        recip = Structure(self._s.lattice.reciprocal_lattice, ["H"], [np.array([0, 0, 0])])
        recip_nn = recip.get_neighbors(recip[0], self._gmax)

        for (n, dist) in recip_nn:
            gvect = n.coords
            gsquare = np.linalg.norm(gvect) ** 2

            expval = exp(-1.0 * gsquare / (4.0 * self._eta))

            gvect_tile = np.tile(gvect, (numsites, 1))
            gvectdot = np.sum(gvect_tile * coords, 1)
            #calculate the structure factor
            sfactor = np.zeros((numsites, numsites))
            sreal = 0.0
            simag = 0.0
            for i in xrange(numsites):
                qi = self._oxi_states[i]
                g_dot_i = gvectdot[i]
                sfactor[i, i] = qi * qi
                sreal += qi * cos(g_dot_i)
                simag += qi * sin(g_dot_i)

                for j in xrange(i + 1, numsites):
                    qj = self._oxi_states[j]
                    exparg = g_dot_i - gvectdot[j]
                    cosa = cos(exparg)
                    sina = sin(exparg)
                    sfactor[i, j] = qi * qj * (cosa + sina)
                    """
                    Uses the property that when sitei and sitej are switched,
                    exparg' == - exparg. This implies 
                    cos (exparg') = cos (exparg) and
                    sin (exparg') = - sin (exparg)
                    
                    Halves all computations.
                    """
                    sfactor[j, i] = qi * qj * (cosa - sina)

            erecip += expval / gsquare * sfactor
            pref = 2 * expval / gsquare * np.array(self._oxi_states)
            factor = prefactor * pref * (sreal * np.sin(gvectdot) - simag * np.cos(gvectdot)) * EwaldSummation.CONV_FACT
            forces += np.tile(factor, (3, 1)).transpose() * gvect_tile

        return (erecip * prefactor * EwaldSummation.CONV_FACT , forces)


    def _calc_real_and_point(self):
        """
        Determines the self energy -(eta/pi)**(1/2) * sum_{i=1}^{N} q_i**2
        
        If cell is charged a compensating background is added (i.e. a G=0 term)
        """
        all_nn = self._s.get_all_neighbors(self._rmax, True)

        forcepf = 2.0 * self._sqrt_eta / sqrt(pi)
        coords = self._coords
        numsites = self._s.num_sites
        ereal = np.zeros((numsites, numsites))
        epoint = np.zeros((numsites))
        forces = np.zeros((numsites, 3))
        for i in xrange(numsites):
            nn = all_nn[i] #self._s.get_neighbors(site, self._rmax)
            qi = self._oxi_states[i]
            epoint[i] = qi * qi
            epoint[i] *= -1.0 * sqrt(self._eta / pi)
            epoint[i] += qi * pi / (2.0 * self._vol * self._eta)  #add jellium term
            for j in range(len(nn)):  #for (nsite, rij)  in nn:
                nsite = nn[j][0]
                rij = nn[j][1]
                qj = compute_average_oxidation_state(nsite)
                erfcval = erfc(self._sqrt_eta * rij)
                ereal[nn[j][2], i] += erfcval * qi * qj / rij
                fijpf = qj / pow(rij, 3) * (erfcval + forcepf * rij * exp(-self._eta * pow(rij, 2)))
                forces[i] += fijpf * (coords[i] - nsite.coords) * qi * EwaldSummation.CONV_FACT

        ereal = ereal * 0.5 * EwaldSummation.CONV_FACT
        epoint = epoint * EwaldSummation.CONV_FACT
        return (ereal, epoint, forces)

    @property
    def eta(self):
        return self._eta

    def __str__(self):
        output = ["Real = " + str(self.real_space_energy)]
        output.append("Reciprocal = " + str(self.reciprocal_space_energy))
        output.append("Point = " + str(self.point_energy))
        output.append("Total = " + str(self.total_energy))
        output.append("Forces:\n" + str(self.forces))
        return "\n".join(output)


class EwaldMinimizer:
    '''
    This class determines the manipulations that will minimize an ewald matrix, 
    given a list of possible manipulations. This class does not perform the 
    manipulations on a structure, but will return the list of manipulations that 
    should be done on one to produce the minimal structure. It returns the 
    manipulations for the n lowest energy orderings. This class should be used 
    to perform fractional species substitution or fractional species removal to 
    produce a new structure. These manipulations create large numbers of 
    candidate structures, and this class can be used to pick out those with the 
    lowest ewald sum.
    
    An alternative (possibly more intuitive) interface to this class is the 
    order disordered structure transformation.
    
    Author - Will Richards
    '''
    
    ALGO_FAST = 0
    ALGO_COMPLETE = 1
    ALGO_BEST_FIRST = 2

    def __init__(self, matrix, m_list, num_to_return = 1, algo = ALGO_FAST):
        '''
        Args:
            matrix:      
                a matrix of the ewald sum interaction energies. This is stored 
                in the class as a diagonally symmetric array and so self._matrix 
                will not be the same as the input matrix
            m_list:
                list of manipulations. each item is of the form 
                (multiplication fraction, number_of_indices, indices, species)
                These are sorted such that the first manipulation contains the 
                most permutations. this is actually evaluated last in the 
                recursion since I'm using pop.
            num_to_return: 
                The minimizer will find the number_returned lowest energy 
                structures. This is likely to return a number of duplicate 
                structures so it may be necessary to overestimate and then 
                remove the duplicates later. (duplicate checking in this 
                process is extremely expensive)
        '''
        self._matrix = copy(matrix)
        for i in range(len(self._matrix)): #make the matrix diagonally symmetric (so matrix[i,:] == matrix[:,j])
            for j in range(i, len(self._matrix)):
                value = (self._matrix[i, j] + self._matrix[j, i]) / 2
                self._matrix[i, j] = value
                self._matrix[j, i] = value
        self._m_list = deepcopy(sorted(m_list, key = lambda x: comb(len(x[2]), x[1]), reverse = True)) #sort the m_list based on number of permutations
        self._current_minimum = float('inf')

        self._output_lists = []
        self._num_to_return = num_to_return
        
        self._algo = algo
        if algo == EwaldMinimizer.ALGO_COMPLETE:
            raise NotImplementedError('Complete algo not yet implemented for EwaldMinimizer')
        self._finished = False #tag that the recurse function looks at at each level. If a method sets this to true it breaks the recursion and stops the search

        self.minimize_matrix()

        self._best_m_list = self._output_lists[0][1]
        self._minimized_sum = self._output_lists[0][0]
        
    def minimize_matrix(self):
        '''
        This method finds and returns the permutations that produce the lowest ewald sum
        calls recursive function to iterate through permutations
        '''
        if self._algo == EwaldMinimizer.ALGO_FAST or self._algo == EwaldMinimizer.ALGO_BEST_FIRST:
            return self._recurse(self._matrix, self._m_list, set(range(len(self._matrix))))


    def add_m_list(self, matrix_sum, m_list):
        '''
        This adds an m_list to the output_lists and updates the current minimum if the list is full.
        '''
        if self._output_lists is None:
            self._output_lists = [[matrix_sum, m_list]]
        else:
            bisect.insort(self._output_lists, [matrix_sum, m_list])
        if self._algo == EwaldMinimizer.ALGO_BEST_FIRST and len(self._output_lists) == self._num_to_return:
            self._finished = True
        if len(self._output_lists) > self._num_to_return:
            self._output_lists.pop()
        if len(self._output_lists) == self._num_to_return:
            self._current_minimum = self._output_lists[-1][0]


    def best_case(self, matrix, manipulation, indices_left):
        '''
        Computes a best case given a matrix and manipulation list. This is only 
        used for when there is only one manipulation left calculating a best 
        case when there are multiple fractions remaining is much more complex 
        (as sorting and dot products have to be done on each row and it just 
        generally scales pretty badly).
        
        Args:
            matrix: 
                the current matrix (with some permutations already performed
            manipulation: 
                (multiplication fraction, number_of_indices, indices, species) 
                describing the manipulation
            indices: 
                set of indices which haven't had a permutation performed on them 
        '''
        indices = list(indices_left.intersection(manipulation[2])) #only look at the indices that are in the manipulation and haven't been used
        fraction = manipulation[0]
        num = manipulation[1]

        sums = [2 * sum(matrix[i, :]) - (1 - fraction) * matrix[i, i] for i in indices]   #compute the sum of each row. the weird prefactor on the self 
                                                                                #interaction term is to make the math work out because it actually 
                                                                                #is multiplied by f**2vrather than f like all the other te
        if fraction < 1:    #we want do the thing that we guess will be most minimizing (to get to a lower current minimum faster). Whether 
                            #we want to use the most positive or negative row depends on whether we're increasing or decreasing that row
            next_index = indices[sums.index(max(sums))]
        else:
            next_index = indices[sums.index(min(sums))]

        interactions = list(matrix[indices, :][:, indices].flatten()) #compute a list of things that we may have double counted

        if fraction <= 1:    #we use the first part of sorted sums. Whether we want highest or lowest depends on the fraction
            sums.sort(reverse = True)
        else:
            sums.sort()
        if 2 * fraction - fraction ** 2 <= 1:
            interactions.sort(reverse = True)
        else:
            interactions.sort()

        #compute a best case using the most minimizing row sums and the most minimizing interactions
        best_case = sum(sum(matrix)) + (fraction - 1) * sum(sums[:num]) + (2 * fraction - fraction ** 2 - 1) * sum(interactions[:num * (num - 1)])

        return best_case, next_index

    def get_next_index(self, matrix, manipulation, indices_left): #returns an index that should have the most negative effect on the matrix sum
        f = manipulation[0]
        indices = list(indices_left.intersection(manipulation[2]))
        sums = [2 * sum(matrix[i, :]) - (1 - f) * matrix[i, i] for i in indices]
        if f < 1:
            next_index = indices[sums.index(max(sums))]
        else:
            next_index = indices[sums.index(min(sums))]

        return next_index

    def _recurse(self, matrix, m_list, indices, output_m_list = []):
        '''
        This method recursively finds the minimal permutations using a binary tree search strategy
        
        Args:
            matrix: 
                The current matrix (with some permutations already performed
            m_list: 
                The list of permutations still to be performed
            indices: 
                Set of indices which haven't had a permutation performed on them
            fast: 
                boolean indicating whether shortcuts by looking at the best case are to be used, the default is true
        
        Returns:
            [minimal value, [list of replacements]]
                Each replacement is a list [index of replaced specie, specie inserted at that index]
        '''
        #check to see if we've found all the solutions that we need
        if self._finished:
            return
        
        #if we're done with the current manipulation, pop it off. 
        while m_list[-1][1] == 0:
            m_list = copy(m_list)
            m_list.pop()
            if not m_list: #if there are no more manipulations left to do check the value
                matrix_sum = sum(sum(matrix))
                if matrix_sum < self._current_minimum:
                    self.add_m_list(matrix_sum, output_m_list)
                return
        
        #if we wont have enough indices left, return
        if m_list[-1][1] > len(indices.intersection(m_list[-1][2])):
            return
        
        #get the next index and best case, if there's just one manipulation left
        index = None
        if len(m_list) == 1 and m_list[-1][1] > 1:
            best_case = self.best_case(copy(matrix), m_list[-1], indices)
            index = best_case[1]
            if best_case[0] > self._current_minimum:
                return
            
        if index is None:
            index = self.get_next_index(matrix, m_list[-1], indices)

        m_list[-1][2].remove(index)
        
        #make the matrix and new m_list where we do the manipulation to the index that we just got
        matrix2 = copy(matrix)
        m_list2 = deepcopy(m_list)
        output_m_list2 = deepcopy(output_m_list)

        matrix2[index, :] *= m_list[-1][0]
        matrix2[:, index] *= m_list[-1][0]
        output_m_list2.append([index, m_list[-1][3]])
        indices2 = copy(indices)
        m_list2[-1][1] -= 1

        #recurse through both the modified and unmodified matrices
        if index in indices2:
            indices2.remove(index)
            self._recurse(matrix2, m_list2, indices2, output_m_list2)
        self._recurse(matrix, m_list, indices, output_m_list)
        

    @property
    def best_m_list(self):
        return self._best_m_list

    @property
    def minimized_sum(self):
        return self._minimized_sum

    @property
    def output_lists(self):
        return self._output_lists


def compute_average_oxidation_state(site):
    """
    Calculates the average oxidation state of a site
    
    Args:
        Site to compute average oxidation state
        
    Returns:
        Average oxidation state of site.
    """
    return sum([sp.oxi_state * occu for sp, occu in site.species_and_occu.items() if sp != None])





#!/usr/bin/env python

"""
This module provides utility classes for string operations.
"""

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2011, The Materials Project"
__version__ = "1.0"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__status__ = "Production"
__date__ = "$Sep 23, 2011M$"


def str_delimited(results, header=None, delimiter="\t"):
    """
    Given a tuple of tuples, generate a delimited string form.
    >>> results = [['a','b','c'],['d','e','f'],[1,2,3]]
    >>> print str_delimited(results,delimiter=',')
    a,b,c
    d,e,f
    1,2,3
    
    Args:
        result: 2d sequence of arbitrary types.
        header: optional header
        
    Returns:
        Aligned string output in a table-like format.
    
    """
    returnstr = ''
    if header != None:
        returnstr += delimiter.join(header) + "\n"
    return returnstr + "\n".join([delimiter.join([str(m) for m in result]) for result in results])


def str_aligned(results, header=None):
    """
    Given a tuple, generate a nicely aligned string form.
    >>> results = [['a','b','cz'],['d','ez','f'],[1,2,3]]
    >>> print str_aligned(results)
    a    b   cz
    d   ez    f
    1    2    3
    
    Args:
        result: 2d sequence of arbitrary types.
        header: optional header
        
    Returns:
        Aligned string output in a table-like format.
    """
    k = list(zip(*results))
    stringlengths = list()
    count = 0
    for i in k:
        colMaxLength = max([len(str(m)) for m in i])
        if header != None:
            colMaxLength = max([len(str(header[count])), colMaxLength])
        stringlengths.append(colMaxLength)
        count += 1
    formatString = "   ".join(["%" + str(d) + "s" for d in stringlengths])
    returnstr = ''
    if header != None:
        header_str = formatString % tuple(header)
        returnstr += header_str + "\n"
        returnstr += "-" * len(header_str) + "\n"

    return returnstr + "\n".join([formatString % tuple(result) for result in results])


def formula_double_format(afloat, ignore_ones=True, tol=1e-8):
    """
    This function is used to make pretty formulas by formatting the amounts.
    Instead of Li1.0 Fe1.0 P1.0 O4.0, you get LiFePO4.
    
    Args:
        afloat:
            a float
        ignore_ones:
            if true, floats of 1 are ignored.
        tol:
            Tolerance to round to nearest int. i.e. 2.0000000001 -> 2
    
    Returns:
        A string representation of the float for formulas.
    """
    if ignore_ones and afloat == 1:
        return ""
    elif abs(afloat - int(afloat)) < tol:
        return str(int(afloat))
    else:
        return str(afloat)

if __name__ == "__main__":
    import doctest
    doctest.testmod()



#!/usr/bin/env python

"""
This module provides core classes needed by all define electronic structure, 
such as the Spin, Orbital, etc.
"""

from __future__ import division

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2011, The Materials Project"
__version__ = "1.0"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__status__ = "Production"
__date__ = "Sep 23, 2011"


from pymatgen.util.decorators import cached_class


class Spin(object):
    """
    Enum type for Spin.  Only up and down.
    Usage: Spin.up, Spin.down.
    """

    @cached_class
    class _SpinImpl(object):
        """
        Internal representation for Spin. Not to be instantiated directly.
        Use Spin enum types. Class is implemented as a cached class for 
        memory efficiency.
        """
        def __init__(self, name):
            self._name = name

        def __int__(self):
            return 1 if self._name == "up" else -1

        def __repr__(self):
            return self._name

        def __eq__(self, other):
            if other == None:
                return False
            return self._name == other._name

        def __hash__(self):
            return self.__int__()

        def __str__(self):
            return self._name

    @staticmethod
    def from_int(i):
        """
        Provides the spin from an int. +1 == Spin.up, -1 == Spin.down.
        
        Args:
            i:
                integer representing direction of spin.
        """
        if i == 1:
            return Spin.up
        elif i == -1:
            return Spin.down
        else:
            raise ValueError("Spin integers must be 1 or -1")

    up = _SpinImpl("up")
    down = _SpinImpl("down")
    all_spins = (up, down)


class Orbital(object):
    """
    Enum type for OrbitalType. Indices are basically the azimutal quantum number, l.
    Design follows somewhat the familiar Java syntax.
    """

    @cached_class
    class _OrbitalImpl(object):
        """
        Internal representation of an orbital.  Do not use directly. 
        Use the Orbital class enum types.  Class is implemented as a cached 
        class for memory efficiency.
        """

        def __init__(self, name, vasp_index):
            self._name = name
            self._vasp_index = vasp_index

        def __int__(self):
            return self._vasp_index

        def __repr__(self):
            return self._name

        def __eq__(self, other):
            if other == None:
                return False
            return self._name == other._name

        def __hash__(self):
            return self.__int__()

        @property
        def orbital_type(self):
            return self._name[0].upper()

        def __str__(self):
            return self._name

    s = _OrbitalImpl("s", 0)
    py = _OrbitalImpl("py", 1)
    pz = _OrbitalImpl("pz", 2)
    px = _OrbitalImpl("px", 3)
    dxy = _OrbitalImpl("dxy", 4)
    dyz = _OrbitalImpl("dyz", 5)
    dz2 = _OrbitalImpl("dz2", 6)
    dxz = _OrbitalImpl("dxz", 7)
    dx2 = _OrbitalImpl("dx2", 8)
    f_3 = _OrbitalImpl("f_3", 9)
    f_2 = _OrbitalImpl("f_2", 10)
    f_1 = _OrbitalImpl("f_1", 11)
    f0 = _OrbitalImpl("f0", 12)
    f1 = _OrbitalImpl("f1", 13)
    f2 = _OrbitalImpl("f2", 14)
    f3 = _OrbitalImpl("f3", 15)

    all_orbitals = (s,
                    py, pz, px,
                    dxy, dyz, dz2, dxz, dx2,
                    f_3, f_2, f_1, f0, f1, f2, f3)

    @staticmethod
    def from_vasp_index(i):
        for orb in Orbital.all_orbitals:
            if int(orb) == i:
                return orb
        raise IndexError("Illegal exceeds supported orbital set")

    @staticmethod
    def from_string(orb_str):
        for orb in Orbital.all_orbitals:
            if str(orb) == orb_str:
                return orb
        raise ValueError("Illegal orbital definition!")




#!/usr/bin/env python

from __future__ import division

'''
Created on Nov 14, 2011
'''

__author__="Shyue Ping Ong"
__copyright__ = "Copyright 2011, The Materials Project"
__version__ = "0.1"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__date__ = "Nov 14, 2011"

import argparse

from pymatgen.io.vaspio import Poscar
from pymatgen.io.cifio import CifParser, CifWriter

parser = argparse.ArgumentParser(description='''Convenient file format convertor. 
Author: Shyue Ping Ong
Version: 1.0
Last updated: Oct 26 2011''')
parser.add_argument('input_file', metavar='input file', type=str, nargs = 1, help='input file')
parser.add_argument('output_file', metavar='output file', type=str, nargs = 1, help='output file')

parser.add_argument('-c', '--conversion', dest='conversion', type=str, nargs = 1, choices=['poscar2cif','cif2poscar'], default='poscar2cif', help='Format conversion desired. ')

args = parser.parse_args()
try:
    if args.conversion[0] == 'poscar2cif':
        p = Poscar.from_file(args.input_file[0])
        w = CifWriter(p.struct)
        w.write_file(args.output_file[0])
    else:
        r = CifParser(args.input_file[0])
        p = Poscar(r.get_structures()[0])
        p.write_file(args.output_file[0])
except Exception as ex:
    print "Error converting file. Are they in the right format?"
    print str(ex)
    


#!/usr/bin/env python

'''
Utilities for manipulating coordinates or list of coordinates
'''

from __future__ import division

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2011, The Materials Project"
__version__ = "0.1"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__date__ = "Nov 27, 2011"

import numpy as np


def in_coord_list(coord_list, coord, **kwargs):
    """
    Tests if a particular coord is within a coord_list using numpy.allclose.
    
    Args:
        coord_list:
            List of coords to test
        coord:
            Specific coordinates
        kwargs:
            keyword arguments supported by numpy.allclose. Please refer to 
            documentation for numpy.allclose.
    """
    for test in coord_list:
        if np.allclose(test, coord, **kwargs):
            return True
    return False


def get_linear_interpolated_value(x_values, y_values, x):
    """
    Returns an interpolated value by linear interpolation between two values.
    This method is written to avoid dependency on scipy, which causes issues on
    threading servers.
    
    Args:
        x_values:
            Sequence of x values.
        y_values:
            Corresponding sequence of y values
        x:
            Get value at particular x
    """
    val_dict = dict(zip(x_values, y_values))
    if x < min(x_values) or x > max(x_values):
        raise ValueError("x is out of range of provided x_values")

    for val in sorted(x_values):
        if val < x:
            x1 = val
        else:
            x2 = val
            break

    y1 = val_dict[x1]
    y2 = val_dict[x2]

    return y1 + (y2 - y1) / (x2 - x1) * (x - x1)



#!/usr/bin/env python

"""
This module provides classes to define everything related to band structures.
"""

__author__ = "Geoffroy Hautier, Shyue Ping Ong, Michael Kocher"
__copyright__ = "Copyright 2012, The Materials Project"
__version__ = "1.0"
__maintainer__ = "Geoffroy Hautier"
__email__ = "geoffroy@uclouvain.be"
__status__ = "Development"
__date__ = "March 14, 2012"


import numpy as np
import math
from pymatgen.core.lattice import Lattice


class Kpoint(object):
    """
    Class to store kpoint objects. A kpoint is defined with a lattice and frac 
    or cartesian coordinates syntax similar than the site object in 
    pymatgen.core.structure.
    """

    def __init__(self, coords, lattice, to_unit_cell=False,
                 coords_are_cartesian=False, label=None):
        self._lattice = lattice
        self._fcoords = lattice.get_fractional_coords(coords) if coords_are_cartesian else coords
        self._label = label

        if to_unit_cell:
            for i in xrange(len(self._fcoords)):
                self._fcoords[i] = self._fcoords[i] - math.floor(self._fcoords[i])

        self._ccoords = lattice.get_cartesian_coords(self._fcoords)

    @property
    def lattice(self):
        """
        The lattice associated with the site.
        """
        return self._lattice

    @property
    def label(self):
        """
        The lattice associated with the site.
        """
        return self._label

    @property
    def frac_coords(self):
        """
        The fractional coordinates of the site as a tuple.
        """
        return np.copy(self._fcoords)

    @property
    def cart_coords(self):
        """
        The cartesian coordinates
        """
        return np.copy(self._ccoords)

    @property
    def a(self):
        """
        Fractional a coordinate
        """
        return self._fcoords[0]

    @property
    def b(self):
        """
        Fractional b coordinate
        """
        return self._fcoords[1]

    @property
    def c(self):
        """
        Fractional c coordinate
        """
        return self._fcoords[2]

    def __str__(self):
        return str(self.frac_coords) + " " + str(self.cart_coords) + " " + str(self.label)

    @property
    def to_dict(self):
        return {'lattice':self.lattice.to_dict, 'fcoords':list(self.frac_coords), 'ccoords':list(self.cart_coords), 'label':self.label}

class BandStructure(object):
    """
    This is the most generic band structure data possible
    it's defined by a list of kpoints + energy and occupancies for each of them
    """

    def __init__(self, kpoints, eigenvals, lattice, efermi, labels_dict={},
                 coords_are_cartesian=False):
        """
        Args: 
            kpoints:
                Sequence of kpoint as numpy arrays, in frac_coords of the 
                lattice given by default
            eigenvals: 
                Sequence of {} with energy and occup keys each element of the 
                array is one "band"
            lattice: 
                The reciprocal lattice
            label_dict:
                (dict) of {} this link a kpoint (in frac coords or cartesian 
                coordinates depending on the coords).
            coords_are_cartesian:
                Whether coordinates are cartesian.
            efermi: 
                fermi energy in eV
        """


        self._efermi = efermi

        #self._structure = structure
        self._lattice_rec = lattice

        self._kpoints = []
        """
        all kpoints, (order matter!)
        """
        self._labels_dict = {}
        """
        label to each kpoint (with kpoint objects.to_dict)
        """
        for k in kpoints:
            #let see if this kpoint has been assigned a label
            label = None
            for c in labels_dict:
                if(np.linalg.norm(k - np.array(labels_dict[c])) < 0.0001):
                    label = c
                    self._labels_dict[label] = Kpoint(k, lattice, label=label, coords_are_cartesian=coords_are_cartesian)
            self._kpoints.append(Kpoint(k, lattice, label=label, coords_are_cartesian=coords_are_cartesian))

        self._bands = eigenvals
        """
        all energy values for each band at the different kpoints
        """
        self._nb_bands = len(eigenvals)
        """
        the number of bands
        """

    @property
    def kpoints(self):
        """
        return the list of kpoints (as kpoint objects) in the band structure
        """
        return self._kpoints

    @property
    def lattice(self):
        """
        return the lattice of the band structure as a lattice object
        """
        return self._lattice_rec

    @property
    def efermi(self):
        """
        return the fermi energy
        """
        return self._efermi

class BandStructureSymmLine(BandStructure):

    """
    This object stores band structures along selected (symmetry) lines in the Brillouin zone
    """

    def __init__(self, kpoints, eigenvals, lattice, efermi, labels_dict, coords_are_cartesian=False):
        BandStructure.__init__(self, kpoints, eigenvals, lattice, efermi, labels_dict, coords_are_cartesian)
        self._distance = []
        self._branches = []
        """
        all branches labels (ex: Gamma-Z, etc...)
        """
        one_group = []
        branches_tmp = []
        #get labels and distance for each kpoint
        previous_kpoint = self._kpoints[0]
        previous_distance = 0.0

        previous_label = self._kpoints[0].label
        for i in range(len(self._kpoints)):
            label = self._kpoints[i].label
            if label != None and previous_label != None:
                self._distance.append(previous_distance)
            else:
                self._distance.append(np.linalg.norm(self._kpoints[i].cart_coords - previous_kpoint.cart_coords) + previous_distance)
            previous_kpoint = self._kpoints[i]
            previous_distance = self._distance[i]
            if label:
                if previous_label:
                    if len(one_group) != 0:
                        branches_tmp.append(one_group)
                    one_group = []
            previous_label = label
            one_group.append(i)

        if len(one_group) != 0:
            branches_tmp.append(one_group)
        #self._branches=branches
        for b in branches_tmp:
            self._branches.append({'start_index':b[0], 'end_index':b[-1], 'name':(self._kpoints[b[0]].label + "-" + self._kpoints[b[-1]].label)})

    def get_branch_name(self, index):
        to_return = []
        for b in self._branches:
                if(b['start_index'] <= index <= b['end_index']):
                    to_return.append(b['name'])
        return to_return

    def get_vbm(self):
        """
        get the valence band minimum (VBM). returns a dictionary with
        'band_index': a list of the indices of the band containing the VBM (please note that you can have several bands 
        sharing the VBM)
        'kpoint_index': the list of indices in self._kpoints for the kpoint vbm. Please note that there can be several
        kpoint_indices relating to the same kpoint (e.g., Gamma can occur at different spots in the band structure line plot)
        'kpoint': the kpoint (as a kpoint object)
        'energy': the energy of the VBM
        """
        max_tmp = -1000.0
        index = None
        for i in range(self._nb_bands):
            for j in range(len(self._kpoints)):
                if self._bands[i]['occup'][j] != 0.0:
                    if(self._bands[i]['energy'][j] > max_tmp):
                        max_tmp = self._bands[i]['energy'][j]
                        index = j
                        kpointvbm = self._kpoints[j]
        list_index_kpoints = []
        if kpointvbm.label != None:
            for i in range(len(self._kpoints)):
                if self._kpoints[i].label == kpointvbm.label:
                    list_index_kpoints.append(i)
        else:
            list_index_kpoints.append(index)
        #get all other bands sharing the vbm
        list_index_band = []
        for i in range(self._nb_bands):
            if math.fabs(self._bands[i]['energy'][index] - max_tmp) < 0.001:
                list_index_band.append(i)
        return {'band_index':list_index_band, 'kpoint_index':list_index_kpoints, 'kpoint':kpointvbm, 'energy':max_tmp}


    def get_cbm(self):
        """
        get the conduction band minimum (CBM). returns a dictionnary with
        'band_index': a list of the indices of the band containing the CBM (please note that you can have several bands
        sharing the CBM)
        'kpoint_index': the list of indices in self._kpoints for the kpoint cbm. Please note that there can be several
        kpoint_indices relating to the same kpoint (e.g., Gamma can occur at different spots in the band structure line plot)
        'kpoint': the kpoint (as kpoint object)
        'energy': the energy of the CBM
        """
        max_tmp = 1000.0
        index = None
        for i in range(self._nb_bands):
            for j in range(len(self._kpoints)):
                if self._bands[i]['occup'][j] == 0.0:
                    if self._bands[i]['energy'][j] < max_tmp:
                        max_tmp = self._bands[i]['energy'][j]
                        index = j
                        kpointcbm = self._kpoints[j]

        list_index_kpoints = []

        if kpointcbm.label != None:
            for i in range(len(self._kpoints)):
                if self._kpoints[i].label == kpointcbm.label:
                    list_index_kpoints.append(i)
        else:
            list_index_kpoints.append(index)
        #get all other bands sharing the vbm
        list_index_band = []
        for i in range(self._nb_bands):
            if math.fabs(self._bands[i]['energy'][index] - max_tmp) < 0.001:
                list_index_band.append(i)
        return {'band_index': list_index_band, 'kpoint_index': list_index_kpoints, 'kpoint': kpointcbm, 'energy': max_tmp}

    def get_band_gap(self):
        """
        get the band gap 
        returns a dictionary with:
        'energy': the band gap energy in eV
        'direct': a boolean telling if the gap is direct (True) or not (False)
        'transition': the kpoint labels of the transition (e.g., \Gamma to X)
        
        TODO: not sure if the direct works, to test!
        
        """
        # this is wrong because of how Fermi level is defined
        #if self.is_metal():
        #    return {'energy':0.0, 'direct':False, 'transition':None}
        cbm = self.get_cbm()
        vbm = self.get_vbm()
        result = dict(direct=False, energy=0.0, transition=None)

        result['energy'] = cbm['energy'] - vbm['energy']

        if cbm['kpoint'].label == vbm['kpoint'].label or np.linalg.norm(cbm['kpoint'].cart_coords - vbm['kpoint'].cart_coords) < 0.01:
            result['direct'] = True
        result['transition'] = '-'.join([str(c.label) if c.label is not None else str(c.frac_coords) for c in [vbm['kpoint'], cbm['kpoint']]])
        return result


    def is_metal(self):
        """
        check if the band structure indicates a metal by looking if the fermi level crosses a band
        """
        # 25meV (kT)
        threshold = 0.025
        gap = self.get_band_gap()
        return True if gap['energy'] < threshold else False

    @property
    def to_dict(self):
        d = {}
        d['lattice_rec'] = self._lattice_rec.to_dict
        d['efermi'] = self._efermi
        d['kpoints'] = []
        #kpoints are not kpoint objects dicts but are frac coords (this makes the dict
        #smaller and avoids the repetition of the lattice
        for k in self._kpoints:
            d['kpoints'].append(k.to_dict['fcoords'])
        d['branches'] = self._branches
        d['bands'] = self._bands
        d['is_metal'] = self.is_metal()
        d['VBM'] = self.get_cbm()['kpoint'].to_dict
        d['CBM'] = self.get_vbm()['kpoint'].to_dict
        d['band_gap'] = self.get_band_gap()
        d['labels_dict'] = {}
        for c in self._labels_dict:
            d['labels_dict'][c] = self._labels_dict[c].to_dict['fcoords']
        return d

    @staticmethod
    def from_dict(d):
        #d=d['band_structure']
        labels_dict = d['labels_dict']
        return BandStructureSymmLine(d['kpoints'], d['bands'], Lattice.from_dict(d['lattice_rec']), d['efermi'], labels_dict)



def get_reconstructed_band_structure(list_bs, efermi):
        """
        this method takes a list of band structure (divided by branches)
        and reconstruct one band structure object from all of them
        """
        if not efermi:
            efermi = sum([b.efermi for b in list_bs]) / len(list_bs)
        kpoints = []
        eigenvals = []
        labels_dict = {}
        rec_lattice = list_bs[0]._lattice_rec
        #lattice=list_bs[0]._lattice
        #efermi=list_bs[0]._efermi
        nb_bands = list_bs[0]._nb_bands
        for bs in list_bs:
            for k in bs._kpoints:
                kpoints.append(k.frac_coords)
            for k, v in bs._labels_dict.iteritems():
                labels_dict[k] = v.frac_coords
        for i in range(nb_bands):
            eigenvals.append({'energy':[], 'occup':[]})
            for bs in list_bs:
                for e in bs._bands[i]['energy']:
                    eigenvals[i]['energy'].append(e)
                for u in bs._bands[i]['occup']:
                    eigenvals[i]['occup'].append(u)
        return BandStructureSymmLine(kpoints, eigenvals, rec_lattice, efermi, labels_dict)



#!/usr/bin/env python

"""
This module provides classes to create phase diagrams.
"""

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2011, The Materials Project"
__version__ = "1.0"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__status__ = "Production"
__date__ = "Sep 23, 2011"

import numpy as np
import logging
import itertools

from scipy.spatial import Delaunay

from pymatgen.core.structure import Composition
from pymatgen.command_line.qhull_caller import qconvex
from pymatgen.phasediagram.entries import GrandPotPDEntry, TransformedPDEntry
from pymatgen.core.periodic_table import Element
from pymatgen.analysis.reaction_calculator import Reaction

logger = logging.getLogger(__name__)

class PhaseDiagram (object):
    '''
    Simple phase diagram class taking in elements and entries as inputs.
    '''
    FORMATION_ENERGY_TOLERANCE = 1e-11

    def __init__(self, entries, elements=None, use_external_qhull=False):
        """
        Standard constructor for phase diagram.
        
        Args:
            entries:
                A list of PDEntry-like objects having an energy, energy_per_atom and composition.
            elements:
                Optional list of elements in the phase diagram. If set to None, the elements are determined from the
                the entries themselves.
        """
        if elements == None:
            elements = set()
            map(elements.update, [entry.composition.elements for entry in entries])
        self._all_entries = entries
        self._elements = tuple(elements)
        self._qhull_data = None
        self._facets = None
        self._qhull_entries = None
        self._stable_entries = None
        self._all_entries_hulldata = None
        self._use_external_qhull = use_external_qhull
        self._make_phasediagram()

    @property
    def all_entries(self):
        """
        All entries submitted to construct the PhaseDiagram.
        Note that this does not mean that all these entries are actually used in the phase diagram.
        """
        return self._all_entries

    @property
    def elements(self):
        """
        Elements in the phase diagram.
        """
        return self._elements

    @property
    def facets(self):
        """
        Facets of the phase diagram in the form of  [[1,2,3],[4,5,6]...]
        """
        return self._facets

    @property
    def qhull_data(self):
        """
        Data used in the convex hull operation. This is essentially a matrix of
        composition data and energy per atom values created from the qhull_entries.
        """
        return self._qhull_data

    @property
    def qhull_entries(self):
        """
        Actual entries used in convex hull. Excludes all positive formation energy entries.
        """
        return self._qhull_entries

    @property
    def unstable_entries(self):
        """
        Entries that are unstable in the phase diagram. Includes positive formation energy entries.
        """
        return [e for e in self.all_entries if e not in self.stable_entries]

    @property
    def stable_entries(self):
        '''
        Returns the stable entries.
        '''
        return self._stable_entries

    @property
    def all_entries_hulldata(self):
        """
        Same as qhull_data, but for all entries rather than just positive formation energy ones.
        """
        return self._all_entries_hulldata

    @property
    def el_refs(self):
        """
        List of elemental references for the phase diagrams. 
        These are typically entries corresponding to the lowest energy element entries.
        """
        return self._el_refs

    def get_form_energy(self, entry):
        '''
        Returns the formation energy for an entry (NOT normalized) from the
        elemental references.
        
        Args:
            entry:
                A PDEntry
        
        Returns:
            Formation energy from the elementals references.
        '''
        comp = entry.composition
        energy = entry.energy - sum([comp[el] * self._el_refs[el].energy_per_atom for el in comp.elements])
        return energy

    def get_form_energy_per_atom(self, entry):
        '''
        Returns the formation energy per atom for an entry from the
        elemental references.
        '''
        comp = entry.composition
        return self.get_form_energy(entry) / comp.num_atoms

    def _process_entries_qhulldata(self, entries_to_process):
        data = list()
        for entry in entries_to_process:
            comp = entry.composition
            energy_per_atom = entry.energy_per_atom
            row = list()
            for i in xrange(1, len(self._elements)):
                row.append(comp.get_atomic_fraction(self._elements[i]))
            row.append(energy_per_atom)
            data.append(row)
        return data

    def _create_convhull_data(self):
        '''
        Make data suitable for convex hull procedure from the list of entries.
        '''
        logger.debug("Creating convex hull data...")
        #Determine the elemental references based on lowest energy for each.
        self._el_refs = dict()
        for entry in self._all_entries:
            if entry.composition.is_element:
                for el in entry.composition.elements:
                    if entry.composition[el] > Composition.amount_tolerance:
                        break
                e_per_atom = entry.energy_per_atom
                if el not in self._el_refs:
                    self._el_refs[el] = entry
                elif self._el_refs[el].energy_per_atom > e_per_atom:
                    self._el_refs[el] = entry
        # Remove positive formation energy entries
        entries_to_process = list()
        for entry in self._all_entries:
            if self.get_form_energy(entry) <= self.FORMATION_ENERGY_TOLERANCE:
                entries_to_process.append(entry)
            else:
                logger.debug("Removing positive formation energy entry {}".format(entry))

        self._qhull_entries = entries_to_process
        return self._process_entries_qhulldata(entries_to_process)

    def _make_phasediagram(self):
        """
        Make the phase diagram.
        """
        stable_entries = set()
        dim = len(self._elements)
        self._qhull_data = self._create_convhull_data()
        if len(self._qhull_data) == dim:
            self._facets = [range(len(self._elements))]
        else:
            if self._use_external_qhull:
                logger.debug("> 4D hull encountered. Computing hull using external qconvex call.")
                self._facets = qconvex(self._qhull_data)
            else:
                logger.debug("Computing hull using scipy.spatial.delaunay")
                delau = Delaunay(self._qhull_data)
                self._facets = delau.convex_hull
            logger.debug("Final facets are\n{}".format(self._facets))

            logger.debug("Removing vertical facets...")
            finalfacets = list()
            for facet in self._facets:
                facetmatrix = np.zeros((len(facet), len(facet)))
                count = 0
                is_element_facet = True
                for vertex in facet:
                    facetmatrix[count] = np.array(self._qhull_data[vertex])
                    facetmatrix[count, dim - 1] = 1
                    count += 1
                    if len(self._qhull_entries[vertex].composition) > 1:
                        is_element_facet = False
                if abs(np.linalg.det(facetmatrix)) > 1e-8 and (not is_element_facet):
                    finalfacets.append(facet)
                else:
                    logger.debug("Removing vertical facet : {}".format(facet))
            self._facets = finalfacets

        for facet in self._facets:
            for vertex in facet:
                stable_entries.add(self._qhull_entries[vertex])
        self._stable_entries = stable_entries
        self._all_entries_hulldata = self._process_entries_qhulldata(self._all_entries)


class GrandPotentialPhaseDiagram (PhaseDiagram):
    '''
    Grand potential phase diagram class taking in elements and entries as inputs.
    '''

    def __init__(self, entries, chempots, elements, use_external_qhull=False):
        """
        Standard constructor for phase diagram.
        
        Args:
            entries:
                A list of PDEntry-like objects having an energy, energy_per_atom and composition.
            chempots:
                A dict of {element: float} to specify the chemical potentials of the open elements.
            elements:
                Optional list of elements in the phase diagram. If set to None, the elements are determined from the
                the entries themselves.
        """
        allentries = list()
        for entry in entries:
            if not (entry.is_element and (entry.composition.elements[0] in chempots)):
                allentries.append(GrandPotPDEntry(entry, chempots))
        self.chempots = chempots
        filteredels = list()
        for el in elements:
            if el not in chempots:
                filteredels.append(el)
        elements = sorted(filteredels)
        super(GrandPotentialPhaseDiagram, self).__init__(allentries, elements, use_external_qhull)


class CompoundPhaseDiagram(PhaseDiagram):
    """
    Experimental feature. Generates phase diagrams from compounds as termninations
    instead of elements.
    """

    def __init__(self, entries, terminal_compositions, use_external_qhull=False):
        entries = get_entries_within_compositional_space(entries, terminal_compositions)
        elset = get_non_coplanar_element_set(entries)
        els = list(elset)
        pentries = get_transformed_entries(entries, els)
        super(CompoundPhaseDiagram, self).__init__(pentries, use_external_qhull=use_external_qhull)


def get_comp_matrix_from_comp(compositions, elements, normalize_row=True):
    """
    Helper function to generates a normalized composition matrix from a list of 
    composition.
    """
    comp_matrix = np.array([[comp.get_atomic_fraction(el) for el in elements] for comp in compositions])
    if not normalize_row:
        return comp_matrix
    factor = np.tile(np.sum(comp_matrix, 1), (len(elements), 1)).transpose()
    return comp_matrix / factor

def get_comp_matrix(entries, elements, normalize_row=True):
    """
    Helper function to generates a normalized composition matrix from a list of 
    composition.
    """
    return get_comp_matrix_from_comp([entry.composition for entry in entries], elements, normalize_row)

def is_coplanar(entries, elements):
    comp_matrix = get_comp_matrix(entries, elements)
    for submatrix in itertools.combinations(comp_matrix, min(len(elements), len(entries))):
        if abs(np.linalg.det(submatrix)) > 1e-5:
            return False
    return True

def get_non_coplanar_element_set(entries):
    elements = set()
    map(elements.update, [entry.composition.elements for entry in entries])
    for i in xrange(len(elements), 1, -1):
        for elset in itertools.combinations(elements, i):
            if not is_coplanar(entries, elset):
                return elset
    return None

def get_transformed_entries(entries, elements):
    comp_matrix = get_comp_matrix(entries, elements)
    newmat = []
    energies = []
    for i in xrange(len(elements)):
        col = comp_matrix[:, i]
        maxval = max(col)
        maxind = list(col).index(maxval)
        newmat.append(comp_matrix[maxind])
        energies.append(entries[i].energy_per_atom)
    invm = np.linalg.inv(np.array(newmat).transpose())
    newentries = []
    for i in xrange(len(entries)):
        entry = entries[i]
        lincomp = np.dot(invm, comp_matrix[i])
        lincomp = np.around(lincomp, 5)
        comp = Composition({Element.from_Z(j + 1):lincomp[j] for j in xrange(len(elements))})
        scaled_energy = entry.energy_per_atom - sum(lincomp * energies)
        newentries.append(TransformedPDEntry(comp, scaled_energy, entry))
    return newentries

def get_entries_within_compositional_space(entries, terminal_compositions):
    newentries = []
    for entry in entries:
        try:
            rxn = Reaction(terminal_compositions, [entry.composition])
            newentries.append(entry)
        except:
            pass
    return newentries





#!/usr/bin/env python

"""
This module provides classes for plotting PhaseDiagram objects.
"""

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2011, The Materials Project"
__version__ = "1.0"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__status__ = "Production"
__date__ = "Sep 23, 2011"

import math
import numpy as np
import re
import itertools

class PDPlotter(object):
    '''
    A plotter class for phase diagrams.
    '''

    def __init__(self, phasediagram):
        """
        Args:
            phasediagram:
                A PhaseDiagram object.
        """
        self._pd = phasediagram
        self._dim = len(self._pd.elements)
        self.lines = uniquelines(self._pd.facets)
        if self._dim < 2 or self._dim > 4:
            raise ValueError("Only 2-4 components supported!")

    @property
    def pd_plot_data(self):
        '''
        Plot data for phase diagram.
        2-comp - Full hull with energies
        3/4-comp - Projection into 2D or 3D gibbs triangle.
        
        Returns:
            (lines, stable_entries, unstable_entries) 
                - lines is a list of list of coordinates for lines in the PD. 
                - stable_entries is a {coordinate : entry} for each stable node in the phase diagram. (Each coordinate can only have one stable phase)
                - unstable_entries is a {entry: coordinates} for all unstable nodes in the phase diagram.
        '''
        pd = self._pd
        entries = pd.qhull_entries
        data = np.array(pd.qhull_data)
        facetlines = self.lines
        lines = list()
        stable_entries = dict()
        for line in facetlines:
            entry1 = entries[line[0]]
            entry2 = entries[line[1]]
            if self._dim == 2:
                x = [data[line[0]][0], data[line[1]][0]]
                y = [pd.get_form_energy_per_atom(entry1), pd.get_form_energy_per_atom(entry2)]
                coord = [x, y]
            elif self._dim == 3:
                coord = triangular_coord(data[line, 0:2])
            else:
                coord = tet_coord(data[line, 0:3])
            lines.append(coord)
            labelcoord = list(zip(*coord))
            stable_entries[labelcoord[0]] = entry1
            stable_entries[labelcoord[1]] = entry2

        allentries = pd.all_entries
        alldata = np.array(pd.all_entries_hulldata)
        unstable_entries = dict()
        stable = pd.stable_entries
        for i in xrange(0, len(allentries)):
            entry = allentries[i]
            if entry not in stable:
                if self._dim == 2:
                    x = [alldata[i][0], alldata[i][0]]
                    y = [pd.get_form_energy_per_atom(entry), pd.get_form_energy_per_atom(entry)]
                    coord = [x, y]
                elif self._dim == 3:
                    coord = triangular_coord([alldata[i, 0:2], alldata[i, 0:2]])
                else:
                    coord = tet_coord([alldata[i, 0:3], alldata[i, 0:3], alldata[i, 0:3]])
                labelcoord = list(zip(*coord))
                unstable_entries[entry] = labelcoord[0]

        return (lines, stable_entries, unstable_entries)

    def show(self):
        """
        Draws the phase diagram using Matplotlib and show it.
        """
        if self._dim < 4:
            self._show_2d_plot()
        elif self._dim == 4:
            self._show_3d_plot()

    def _show_2d_plot(self):
        '''
        Shows the plot using pylab.  Usually I won't do imports in methods,
        but since plotting is a fairly expensive library to load and not all 
        machines have matplotlib installed, I have done it this way.
        '''
        import matplotlib.pyplot as plt
        from matplotlib.font_manager import FontProperties
        (lines, labels, unstable) = self.pd_plot_data
        for x, y in lines:
            plt.plot(x, y, 'bo-', linewidth=3, markeredgecolor='b', markerfacecolor='r', markersize=10)
        font = FontProperties()
        font.set_weight('bold')
        font.set_size(20)
        count = 1

        if len(self._pd.elements) == 3:
            plt.axis('equal')
            plt.xlim((-0.1, 1.2))
            plt.ylim((-0.1, 1.0))
            plt.axis('off')
            legendstart = [1.0, 0.55]
        else:
            plt.xlim((-0.1, 1.4))
            legendstart = [1.1, 0.0]

        for coords in sorted(labels.keys()):
            entry = labels[coords]
            label = entry.name
            x = coords[0]
            if coords[0] >= math.sqrt(3) / 2:
                halign = 'left'
                x += 0.02
            else:
                halign = 'right'
                x += -0.02
            if coords[1] > 0:
                valign = 'bottom'
            else:
                valign = 'top'

            if len(entry.composition.elements) == 1:
                plt.text(x, coords[1], label, horizontalalignment=halign, verticalalignment=valign, fontproperties=font)
            else:
                plt.text(x, coords[1], str(count), horizontalalignment=halign, verticalalignment=valign, fontproperties=font)
                plt.text(legendstart[0], legendstart[1] - 0.05 * count, str(count) + " : " + label, horizontalalignment='left', verticalalignment='top', fontproperties=font)
                count += 1

        for entry, coords in unstable.items():
            label = entry.name
            plt.plot(coords[0], coords[1], 'bx', linewidth=3, markeredgecolor='b', markerfacecolor='b', markersize=10)

        F = plt.gcf()
        F.set_size_inches((8, 6.4))
        plt.show()

    def _show_3d_plot(self):
        '''
        Shows the plot using pylab.  Usually I won't do imports in methods,
        but since plotting is a fairly expensive library to load and not all 
        machines have matplotlib installed, I have done it this way.
        '''
        import matplotlib.pyplot as plt
        import mpl_toolkits.mplot3d.axes3d as p3
        from matplotlib.font_manager import FontProperties
        fig = plt.figure()
        ax = p3.Axes3D(fig)
        font = FontProperties()
        font.set_weight('bold')
        font.set_size(20)
        (lines, labels, unstable) = self.pd_plot_data
        count = 1
        newlabels = list()
        for x, y, z in lines:
            ax.plot(x, y, z, 'bo-', linewidth=3, markeredgecolor='b', markerfacecolor='r', markersize=10)
        for coords in sorted(labels.keys()):
            entry = labels[coords]
            label = entry.name
            if len(entry.composition.elements) == 1:
                # commented out options are only for matplotlib 1.0.  Removed them so that most ppl can use this class.
                ax.text(coords[0], coords[1], coords[2], label)#, horizontalalignment=halign, verticalalignment=valign, fontproperties=font)
            else:
                ax.text(coords[0], coords[1], coords[2], str(count))#, horizontalalignment=halign, verticalalignment=valign, fontproperties=font)
                newlabels.append(str(count) + " : " + label)
                count += 1
        plt.figtext(0.01, 0.01, '\n'.join(newlabels))
        ax.axis('off')
        plt.show()

    def write_image(self, stream, image_format="svg"):
        '''
        Writes the phase diagram to an image in a stream.
        
        Args:
            stream:
                stream to write to. Can be a file stream or a StringIO stream.
            image_format
                format for image. Can be any of matplotlib supported formats. Defaults to svg for best results for vector graphics.
        '''
        (lines, labels, unstable) = self.pd_plot_data
        dim = len(self._pd.elements)
        elementref = re.compile("^[A-Z][a-z]*$")
        count = 1
        import matplotlib as mpl
        from matplotlib.font_manager import FontProperties

        # chose a non-GUI backend
        mpl.use('Agg')
        import matplotlib.pyplot as plt
        font = FontProperties()
        font.set_weight('bold')
        font.set_size(20)

        if dim == 4:
            plt.clf()
            plt.cla()
            import mpl_toolkits.mplot3d.axes3d as p3
            fig = plt.figure()
            ax = p3.Axes3D(fig)

            newlabels = list()
            for x, y, z in lines:
                ax.plot(x, y, z, 'bo-', linewidth=4, markeredgecolor='b', markerfacecolor='r', markersize=12)
            for coords in sorted(labels.keys()):
                label = labels[coords].name
                if elementref.match(label):
                    ax.text(coords[0], coords[1], coords[2], label, fontproperties=font)
                else:
                    ax.text(coords[0], coords[1], coords[2], str(count), fontproperties=font)
                    newlabels.append(str(count) + " : " + label)
                    count += 1
            plt.figtext(0.01, 0.01, '\n'.join(newlabels), fontproperties=font)

        elif dim < 4 and dim > 1:
            plt.clf()
            plt.cla()

            for x, y in lines:
                plt.plot(x, y, 'bo-', linewidth=4, markeredgecolor='b', markerfacecolor='r', markersize=12)
            if dim == 3:
                plt.axis('equal')
                plt.xlim((-0.02, 1.18))
                plt.ylim((-0.1, 1.0))
                plt.axis('off')
                legendstart = [1.0, 1.0]
                legendspacing = 0.05
            else:
                plt.xlim((-0.1, 1.4))
                legendstart = [1.1, 0.0]
            ymin, ymax = plt.ylim()
            legendspacing = (ymax - ymin) / len(labels)

            for coords in sorted(labels.keys()):
                label = labels[coords].name
                x = coords[0]
                if coords[0] >= math.sqrt(3) / 2:
                    halign = 'left'
                    x += 0.02
                else:
                    halign = 'right'
                    x += -0.02
                if coords[1] > 0:
                    valign = 'bottom'
                else:
                    valign = 'top'

                if elementref.match(label):
                    plt.text(x, coords[1], label, horizontalalignment=halign, verticalalignment=valign, fontproperties=font)
                else:
                    plt.text(x, coords[1], str(count), horizontalalignment=halign, verticalalignment=valign, fontproperties=font)
                    plt.text(legendstart[0], legendstart[1] - legendspacing * count, str(count) + " : " + label, horizontalalignment='left', verticalalignment='top', fontproperties=font)
                    count += 1
        f = plt.gcf()
        f.set_size_inches((12, 10))

        plt.savefig(stream, format=image_format)

def uniquelines(q):
    '''
    Given all the facets, convert it into a set of unique lines.  Specifically used for converting convex hull facets into line pairs of coordinates.
    
    Args:
        q:
            A 2-dim sequence, where each row represents a facet. E.g., [[1,2,3],[3,6,7],...]
    
    Returns:
        setoflines:
            A set of tuple of lines.  E.g., {(1,2), (1,3), (2,3), ....}
    '''
    setoflines = set()
    for facets in q:
        for line in itertools.combinations(facets, 2):
            setoflines.add(tuple(line))
    return setoflines


def triangular_coord(coord):
    '''
    Convert a two component coordinate into a triangle based coordinate system
    for a prettier phase diagram.
    
    Args:
        coordinate:
            coordinate used in the convex hull computation.
    
    Returns:
        coordinates in a triangular-based coordinate system. 
    '''
    unitvec = np.array([[1, 0], [0.5, math.sqrt(3) / 2]])
    result = np.dot(np.array(coord), unitvec)
    return result.transpose()

def tet_coord(coord):
    '''
    Convert a four component coordinate into a tetrahedron based coordinate system
    for a prettier phase diagram.
    
    Args:
        coordinate:
            coordinate used in the convex hull computation.
    
    Returns:
        coordinates in a tetrahedron-based coordinate system. 
    '''
    unitvec = np.array([[1, 0, 0], [0.5, math.sqrt(3) / 2, 0], [0.5, 1.0 / 3.0 * math.sqrt(3) / 2, math.sqrt(6) / 3]])
    result = np.dot(np.array(coord), unitvec)
    return result.transpose()



#!/usr/bin/env python

'''
This module contains useful decorators for a variety of functions.
'''

from __future__ import division

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2011, The Materials Project"
__version__ = "0.1"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__date__ = "Dec 31, 2011"

import logging
import datetime
from functools import wraps


def singleton(cls):
    """
    This decorator can be used to create a singleton out of a class.
    """

    instances = {}

    def getinstance():
        if cls not in instances:
            instances[cls] = cls()
        return instances[cls]
    return getinstance


def cached_class(klass):
    """
    Decorator to cache class instances by constructor arguments.
    This results in a class that behaves like a singleton for each
    set of constructor arguments, ensuring efficiency. 
    
    Note that this should be used for *immutable classes only*.  Having 
    a cached mutable class makes very little sense.  For efficiency,
    avoid using this decorator for situations where there are many 
    constructor arguments permutations.
      
    The keywords argument dictionary is converted to a tuple because
    dicts are mutable; keywords themselves are strings and 
    so are always hashable, but if any arguments (keyword
    or positional) are non-hashable, that set of arguments
    is not cached.
    """
    cache = {}

    @wraps(klass, assigned=('__name__', '__module__'), updated=())
    class _decorated(klass):
        # The wraps decorator can't do this because __doc__
        # isn't writable once the class is created
        __doc__ = klass.__doc__
        def __new__(cls, *args, **kwds):
            key = (cls,) + args + tuple(kwds.items())
            if key not in cache:
                o = super(klass, cls).__new__(cls, *args, **kwds)
                cache[key] = o
            return cache[key]
    return _decorated


def logged(level=logging.DEBUG):
    """
    Useful logging decorator. If a method is logged, the beginning and end of 
    the method call will be logged at a pre-specified level.
    
    Args:
        level:
            Level to log method at. Defaults to DEBUG.
    """
    def wrap(f):
        logger = logging.getLogger("{}.{}".format(f.__module__, f.__name__))
        def wrapped_f(*args, **kwargs):

            logger.log(level, "Called at {} with args = {} and kwargs = {}".format(datetime.datetime.now(), args, kwargs))
            data = f(*args, **kwargs)
            logger.log(level, "Done at {} with args = {} and kwargs = {}".format(datetime.datetime.now(), args, kwargs))
            return data
        return wrapped_f
    return wrap



#!/usr/bin/env python

'''
Simple structure visualizer script.
'''

from __future__ import division

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2011, The Materials Project"
__version__ = "0.1"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__date__ = "Nov 29, 2011"

import argparse

from pymatgen.io.vaspio import Poscar
from pymatgen.io.cifio import CifParser
from pymatgen.vis.structure_vtk import StructureVis

parser = argparse.ArgumentParser(description='''Convenient structure file viewer.  Currently only cif and POSCAR formats supported.
Author: Shyue Ping Ong
Version: 1.0
Last updated: Nov 29 2011''')
parser.add_argument('input_file', metavar='input file', type=str, nargs=1, help='input file')

parser.add_argument('-f', '--format', dest='format', type=str, nargs=1, choices=['cif', 'poscar'], default='poscar', help='Format of input file. Defaults to POSCAR')
parser.add_argument('-e', '--exclude_bonding', dest='exclude_bonding', type=str, nargs=1, help='List of elements to exclude from bonding analysis. E.g., Li,Na')

args = parser.parse_args()
excluded_bonding_elements = args.exclude_bonding[0].split(',') if args.exclude_bonding else []

file_format = args.format

if args.input_file[0].endswith(".cif"):
    file_format = "cif"
elif args.input_file[0].startswith("POSCAR"):
    file_format = "poscar"

s = None

if file_format == 'poscar':
    p = Poscar.from_file(args.input_file[0])
    s = p.struct
else:
    r = CifParser(args.input_file[0])
    s = r.get_structures(False)[0]

if s:
    vis = StructureVis(excluded_bonding_elements=excluded_bonding_elements)
    vis.set_structure(s)
    vis.show()




#!/usr/bin/env python

"""
This module defines classes to represent the density of states, etc.
"""

from __future__ import division

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2012, The Materials Project"
__version__ = "2.0"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__date__ = "Mar 20, 2012"

from collections import OrderedDict

import numpy as np

from pymatgen.electronic_structure.core import Spin, Orbital
from pymatgen.core.structure import Structure
from pymatgen.util.plotting_utils import get_publication_quality_plot
from pymatgen.util.io_utils import clean_json
from pymatgen.util.coord_utils import get_linear_interpolated_value


class Dos(object):
    """
    Basic DOS object. All other DOS objects are extended versions of this object.
    """

    def __init__(self, efermi, energies, densities):
        """
        Args:
            efermi:
                Fermi level energy
            energies:
                A sequences of energies
            densities:
                A dict of {Spin: np.array} representing the density of states 
                for each Spin.
        """
        self._efermi = efermi
        self._energies = np.array(energies)
        self._dos = {k:np.array(d) for k, d in densities.items()}

    def get_densities(self, spin=None):
        """
        Returns the density of states for a particular spin. If Spin is None,
        the sum of both Spin.up and Spin.down (if they exist) is returned.
        
        Args:
            spin:
                Spin
        """
        if self._dos == None:
            result = None
        elif spin == None:
            if Spin.down in self._dos:
                result = self._dos[Spin.up] + self._dos[Spin.down]
            else:
                result = self._dos[Spin.up]
        else:
            result = self._dos[spin]
        return result

    @property
    def densities(self):
        """
        Dict representation of the densities, {Spin: densities}
        """
        return self._dos

    def get_smeared_densities(self, sigma):
        """
        Returns the Dict representation of the densities, {Spin: densities}, 
        but with a Gaussian smearing of std dev sigma applied about the fermi
        level.
        
        Args:
            sigma:
                Std dev of Gaussian smearing function.
        """
        from scipy.ndimage.filters import gaussian_filter1d
        smeared_dens = {}
        diff = [self._energies[i + 1] - self._energies[i] for i in xrange(len(self.energies) - 1)]
        avgdiff = sum(diff) / len(diff)
        for spin, dens in self._dos.items():
            smeared_dens[spin] = gaussian_filter1d(dens, sigma / avgdiff)
        return smeared_dens

    @property
    def efermi(self):
        """
        Fermi energy
        """
        return self._efermi

    @property
    def energies(self):
        """
        Energy levels of the DOS.
        """
        return self._energies

    def __add__(self, other):
        """
        Adds two DOS together. Checks that energy scales are the same. 
        Otherwise, a ValueError is thrown.
        
        Args:
            other:
                Another DOS object.
                
        Returns:
            Sum of the two DOSs.
        """
        if not (self.energies == other.energies).all():
            raise ValueError("Energies of both DOS are not compatible!")
        densities = {spin: self._dos[spin] + other._dos[spin] for spin in self._dos.keys()}
        return Dos(self.efermi, self.energies, densities)

    def get_interpolated_value(self, energy):
        """
        Returns interpolated density for a particular energy.
        
        Args:
            energy:
                Energy to return the density for.
        """
        f = {}
        for spin in self._dos.keys():
            f[spin] = get_linear_interpolated_value(self._energies, self._dos[spin], energy)
        return f

    def get_interpolated_gap(self, tol=0.001, abs_tol=False, spin=None):
        """
        Expects a DOS object and finds the gap
        
        Args:
            tol:
                tolerance in occupations for determining the gap
            abs_tol:
                tolerance an absolute tolerance (True) and a relative one (False)
            spin:
                Possible values are:
                    None - finds the ap in the summed densities
                    Up - finds the gap in the up spin channel
                    Down - finds the gap in teh down spin channel
        
        Returns:
            (gap, cbm, vbm) :  tuple of floats in eV corresponding to the gap, cbm and vbm
        """

        tdos = self.get_densities(spin)
        if abs_tol == False:
            tol = tol * tdos.sum() / tdos.shape[0]
        energies = self._energies
        below_fermi = [i for i in xrange(len(energies)) if energies[i] < self._efermi and tdos[i] > tol]
        above_fermi = [i for i in xrange(len(energies)) if energies[i] > self._efermi and tdos[i] > tol]
        vbm_start = max(below_fermi)
        cbm_start = min(above_fermi)
        if vbm_start == cbm_start:
            return 0.0, self._efermi, self._efermi
        else:
            # Interpolate between adjacent values
            start = get_linear_interpolated_value(tdos[vbm_start:vbm_start + 2][::-1], energies[vbm_start:vbm_start + 2][::-1], tol)
            end = get_linear_interpolated_value(tdos[cbm_start - 1:cbm_start + 1], energies[cbm_start - 1:cbm_start + 1], tol)
            return end - start, end, start

    def get_cbm_vbm(self, tol=0.001, abs_tol=False, spin=None):
        """
        Expects a DOS object and finds the cbm and vbm.
        
        Args:
            tol: 
                tolerance in occupations for determining the gap
            abs_tol: 
                an absolute tolerance (True) and a relative one (False)
            spin:
                Possible values are:
                    None - finds the gap in the summed densities
                    Up - finds the gap in the up spin channel
                    Down - finds the gap in teh down spin channel
        
        Returns:
            (cbm, vbm): float in eV corresponding to the gap
        """
        #determine tolerance
        tdos = self.get_densities(spin)
        if abs_tol == False:
            tol = tol * tdos.sum() / tdos.shape[0]

        # find index of fermi energy
        i_fermi = 0
        while (self._energies[i_fermi] <= self._efermi):
            i_fermi += 1

        # work backwards until tolerance is reached
        i_gap_start = i_fermi
        while i_gap_start - 1 >= 0 and tdos[i_gap_start - 1] <= tol :
            i_gap_start -= 1

        # work forwards until tolerance is reached
        i_gap_end = i_gap_start
        while i_gap_end < tdos.shape[0] and tdos[i_gap_end] <= tol :
            i_gap_end += 1
        i_gap_end -= 1
        return (self._energies[i_gap_end], self._energies[i_gap_start])

    def get_gap(self, tol=0.001, abs_tol=False, spin=None):
        """
        Expects a DOS object and finds the gap.
        
        Args:
            tol: 
                tolerance in occupations for determining the gap
            abs_tol: 
                an absolute tolerance (True) and a relative one (False)
            spin:
                Possible values are:
                    None - finds the gap in the summed densities
                    Up - finds the gap in the up spin channel
                    Down - finds the gap in teh down spin channel
        
        Returns:
            gap in eV
        """
        (cbm, vbm) = self.get_cbm_vbm(tol, abs_tol, spin)
        return max(cbm - vbm, 0.0)

    def __str__(self):
        """
        Returns a string which can be easily plotted (using gnuplot).
        """
        if Spin.down in self._dos:
            stringarray = ["#%30s %30s %30s" % ('Energy', 'DensityUp', 'DensityDown')]
            stringarray.extend(["%.5f %.5f %.5f" % (self._energies[i], self._dos[Spin.up][i], self._dos[Spin.down][i]) for i in range(len(self._energies))])
        else:
            stringarray = ["#%30s %30s" % ('Energy', 'DensityUp')]
            stringarray.extend(["%.5f %.5f" % (self._energies[i], self._dos[Spin.up][i]) for i in range(len(self._energies))])
        return "\n".join(stringarray)

    @staticmethod
    def from_dict(d):
        """
        Returns Dos object from dict representation of Dos.
        """
        return Dos(d['efermi'], d['energies'], { Spin.from_int(int(k)):v for k, v in d['densities'].items()})

    @property
    def to_dict(self):
        """
        Json-serializable dict representation of Dos.
        """
        d = {}
        d['efermi'] = self._efermi
        d['energies'] = list(self._energies)
        d['densities'] = { str(int(spin)) : list(dens) for spin , dens in self._dos.items() }
        return d


class PDos(Dos):
    """
    Projected DOS for a specific orbital. Extends the Dos object.
    """
    def __init__(self, efermi, energies, densities, orbital):
        """
        Args:
            efermi:
                Fermi level energy
            energies:
                A sequences of energies
            densities:
                A dict of {Spin: np.array} representing the density of states 
                for each Spin.
            site:
                Site associated with the projected DOS.
            orbital:
                The orbital associated with the projected DOS.
        """
        Dos.__init__(self, efermi, energies, densities)
        self.orbital = orbital

    def __str__(self):
        return "#" + str(self.orbital) + "\n" + super(PDos, self).__str__()

    @staticmethod
    def from_dict(d):
        """
        Returns PDos object from dict representation.
        """
        return PDos(d['efermi'], d['energies'], { Spin.from_int(int(k)):v for k, v in d['densities'].items()}, Orbital.from_string(d['orbital']))

    @property
    def to_dict(self):
        """
        Json-serializable dict representation of PDos.
        """
        d = {}
        d['efermi'] = self._efermi
        d['energies'] = list(self._energies)
        d['densities'] = { str(int(spin)) : list(dens) for spin , dens in self._dos.items() }
        d['orbital'] = str(self.orbital)
        return d

class CompleteDos(Dos):
    """
    This wrapper class defines a total dos, and also provides a list of PDos.
    Mainly used by pymatgen.io.vaspio.Vasprun to create a complete Dos from
    a vasprun.xml file. You are unlikely to try to generate this object manually.
    """

    def __init__(self, structure, total_dos, pdoss):
        """
        Args:
            structure:
                Structure associated with this particular DOS.
            total_dos:
                total Dos for structure
            pdoss:
                The pdoss are supplied as an { Site : {Orbital : Dos.. }}
        """
        self._efermi = total_dos.efermi
        self._energies = total_dos.energies
        self._dos = total_dos.densities
        self._pdos = pdoss
        self._structure = structure

    @property
    def structure(self):
        """
        Structure associated with the CompleteDos.
        """
        return self._structure

    def get_site_orbital_dos(self, site, orbital):
        """
        Get the Dos for a particular orbital of a particular site.
        
        Args:
            site:
                Site in Structure associated with CompleteDos.
            orbital:
                Orbital in the site.
                
        Returns:
            Dos containing densities for orbital of site.
        """
        return self._pdos[site][orbital]

    def get_site_dos(self, site):
        """
        Get the total Dos for a site (all orbitals).
        
        Args:
            site:
                Site in Structure associated with CompleteDos.
                
        Returns:
            Dos containing summed orbital densities for site.
        """
        site_dos = None
        for pdos in self._pdos[site].values():
            if site_dos == None:
                site_dos = Dos(pdos.efermi, pdos.energies, pdos.densities)
            else:
                site_dos += pdos
        return site_dos

    def get_spd_dos(self):
        """
        Get orbital projected Dos.
        
        Returns:
            dict of {orbital: Dos}, e.g. {'s': Dos object, ...}
        """
        spd_dos = dict()
        for atom_dos in self._pdos.values():
            for pdos in atom_dos.values():
                orbital_type = pdos.orbital.orbital_type
                if orbital_type not in spd_dos:
                    spd_dos[orbital_type] = Dos(pdos.efermi, pdos.energies, pdos.densities)
                else:
                    spd_dos[orbital_type] += pdos
        return spd_dos

    def get_element_dos(self):
        """
        Get element projected Dos.
        
        Returns:
            dict of {Element: Dos}
        """

        el_dos = dict()
        for site, atom_dos in self._pdos.items():
            el = site.specie
            for pdos in atom_dos.values():
                if el not in el_dos:
                    el_dos[el] = Dos(pdos.efermi, pdos.energies, pdos.densities)
                else:
                    el_dos[el] += pdos
        return el_dos

    @staticmethod
    def from_dict(d):
        """
        Returns CompleteDos object from dict representation.
        """
        tdos = Dos.from_dict(d)
        struct = Structure.from_dict(d['structure'])
        pdoss = {}
        for i in xrange(len(d['pdos'])):
            at = struct[i]
            orb_dos = {}
            for orb_str, odos in d['pdos'][i].items():
                orb = Orbital.from_string(orb_str)
                orb_dos[orb] = PDos(odos['efermi'], odos['energies'], { Spin.from_int(int(k)):v for k, v in odos['densities'].items()}, orb)
            pdoss[at] = orb_dos
        return CompleteDos(struct, tdos, pdoss)

    @property
    def to_dict(self):
        """
        Json-serializable dict representation of CompleteDos.
        """
        d = {}
        d['efermi'] = self._efermi
        d['structure'] = self._structure.to_dict
        d['energies'] = list(self._energies)
        d['densities'] = { str(int(spin)) : list(dens) for spin , dens in self._dos.items() }
        d['pdos'] = []
        if len(self._pdos) > 0:
            for at in self._structure:
                dd = dict()
                for pdos in self._pdos[at].values():
                    dd[str(pdos.orbital)] = {'efermi' : pdos.efermi, 'energies': list(pdos.energies), 'densities' : { str(int(spin)) : list(dens) for spin , dens in pdos.densities.items() }}
                d['pdos'].append(dd)
            d['atom_dos'] = {str(at) : dos.to_dict for at, dos in self.get_element_dos().items()}
            d['spd_dos'] = {str(orb) : dos.to_dict for orb, dos in self.get_spd_dos().items()}
        return d

    def __str__(self):
        return "Complete DOS for " + str(self._structure)


class DosPlotter(object):
    """
    Class for plotting DOSes.
    """

    def __init__(self, zero_at_efermi=True, stack=False,
                 sigma=None):
        """
        Args:
            zero_at_efermi:
                Whether to shift all Dos to have zero energy at the fermi energy.
                Defaults to True.
            stack:
                Whether to plot the DOS as a stacked area graph
            key_sort_func:
                function used to sort the dos_dict keys.
            sigma:
                A float specifying a standard deviation for Gaussian smearing the 
                DOS for nicer looking plots. Defaults to None for no smearing.
        """
        self.zero_at_efermi = zero_at_efermi
        self.stack = stack
        self.sigma = sigma
        self._doses = OrderedDict()

    def add_dos(self, label, dos):
        """
        Adds a dos for plotting.
        
        Args:
            label:
                label for the DOS. Must be unique.
            dos:
                Dos object
        """
        energies = dos.energies - dos.efermi if self.zero_at_efermi else dos.energies
        densities = dos.get_smeared_densities(self.sigma) if self.sigma else dos.densities
        efermi = dos.efermi
        self._doses[label] = {'energies':energies, 'densities': densities, 'efermi':efermi}

    def add_dos_dict(self, dos_dict, key_sort_func=None):
        """
        Add a dictionary of doses, with an optional sorting function for the 
        keys.
        
        Args:
            dos_dict:
                dict of {label: Dos}
            key_sort_func:
                function used to sort the dos_dict keys.
        """
        if key_sort_func:
            keys = sorted(dos_dict.keys(), key=key_sort_func)
        else:
            keys = dos_dict.keys()
        for label in keys:
            self.add_dos(label, dos_dict[label])

    def get_dos_dict(self):
        """
        Returns the added doses as a json-serializable dict. Note that if you
        have specified smearing for the DOS plot, the densities returned will
        be the smeared densities, not the original densities.
        
        Returns:
            Dict of dos data. Generally of the form, {label: {'energies':..,
            'densities': {'up':...}, 'efermi':efermi}}
        """
        return clean_json(self._doses)

    def show(self, xlim=None, ylim=None):
        """
        Show the plot using matplotlib.
        
        Args:
            xlim:
                Specifies the x-axis limits. Set to None for automatic 
                determination.
            ylim:
                Specifies the y-axis limits. 
        """
        plt = get_publication_quality_plot(12, 8)
        color_order = ['r', 'b', 'g', 'c']

        y = None
        alldensities = []
        allenergies = []
        """
        Note that this complicated processing of energies is to allow for
        stacked plots in matplotlib.
        """
        for key, dos in self._doses.items():
            energies = dos['energies']
            densities = dos['densities']
            if not y:
                y = {Spin.up: np.zeros(energies.shape), Spin.down: np.zeros(energies.shape)}
            newdens = {}
            for spin in [Spin.up, Spin.down]:
                if spin in densities:
                    if self.stack:
                        y[spin] += densities[spin]
                        newdens[spin] = y[spin].copy()
                    else:
                        newdens[spin] = densities[spin]
            allenergies.append(energies)
            alldensities.append(newdens)

        keys = list(self._doses.keys())
        keys.reverse()
        alldensities.reverse()
        allenergies.reverse()
        allpts = []
        for i, key in enumerate(keys):
            x = []
            y = []
            for spin in [Spin.up, Spin.down]:
                if spin in alldensities[i]:
                    densities = list(int(spin) * alldensities[i][spin])
                    energies = list(allenergies[i])
                    if spin == Spin.down:
                        energies.reverse()
                        densities.reverse()
                    x.extend(energies)
                    y.extend(densities)
            allpts.extend(zip(x, y))
            if self.stack:
                plt.fill(x, y, color=color_order[i % 4], label=str(key))
            else:
                plt.plot(x, y, color=color_order[i % 4], label=str(key))
            if not self.zero_at_efermi:
                ylim = plt.ylim()
                plt.plot([self._doses[key]['efermi'], self._doses[key]['efermi']], ylim, color_order[i % 4] + '--', linewidth=2)

        plt.xlabel('Energies (eV)')
        plt.ylabel('Density of states')
        if xlim:
            plt.xlim(xlim)
        if ylim:
            plt.ylim(ylim)
        else:
            xlim = plt.xlim()
            relevanty = [p[1] for p in allpts if p[0] > xlim[0] and p[0] < xlim[1]]
            plt.ylim((min(relevanty), max(relevanty)))

        if self.zero_at_efermi:
            ylim = plt.ylim()
            plt.plot([0, 0], ylim, 'k--', linewidth=2)

        plt.legend()
        leg = plt.gca().get_legend()
        ltext = leg.get_texts()  # all the text.Text instance in the legend
        plt.setp(ltext, fontsize=30)
        plt.tight_layout()
        plt.show()




#!/usr/bin/env python

'''
This module provides input and output from the CSSR file format.
'''

from __future__ import division

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2012, The Materials Project"
__version__ = "0.1"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__date__ = "Jan 24, 2012"


class Cssr(object):
    """
    Basic object for working with Cssr file. Right now, only conversion from 
    a Structure to a Cssr file is supported.
    """

    def __init__(self, structure):
        if not structure.is_ordered:
            raise ValueError("Cssr file can only be constructed from ordered structure")
        self._structure = structure

    def __str__(self):
        output = []
        output.append("{:.4f} {:.4f} {:.4f}".format(*self._structure.lattice.abc))
        output.append("{:.2f} {:.2f} {:.2f} SPGR =  1 P 1    OPT = 1".format(*self._structure.lattice.angles))
        output.append("{} 0".format(len(self._structure)))
        output.append("0 {}".format(self._structure.formula))
        for i, site in enumerate(self._structure.sites):
            output.append("{} {} {:.4f} {:.4f} {:.4f}".format(i + 1, site.specie, site.a, site.b, site.c))
        return "\n".join(output)

    def write_file(self, filename):
        with open(filename, 'w') as f:
            f.write(str(self) + "\n")

